@@ -9,7 +9,7 @@ import sys
 from . import config, errors, runtests, types
 
 # Re-export typeof
-from .special import typeof
+from .special import typeof, prange, pndindex
 
 # Re-export error classes
 from .errors import *
@@ -20,7 +20,7 @@ from .types import *
 from .smartarray import SmartArray
 
 # Re-export decorators
-from .decorators import autojit, cfunc, generated_jit, jit, njit
+from .decorators import autojit, cfunc, generated_jit, jit, njit, stencil
 
 # Re-export vectorize decorators
 from .npyufunc import vectorize, guvectorize
@@ -43,13 +43,16 @@ __all__ = """
     jit
     jitclass
     njit
+    stencil
     typeof
+    prange
+    stencil
     vectorize
     """.split() + types.__all__ + errors.__all__
 
 
-_min_llvmlite_version = (0, 9, 0)
-_min_llvm_version = (3, 7, 0)
+_min_llvmlite_version = (0, 22, 0)
+_min_llvm_version = (5, 0, 0)
 
 def _ensure_llvm():
     """
