@@ -12,7 +12,6 @@ import itertools
 from types import CodeType, ModuleType
 
 from numba import errors, utils
-from numba.config import PYVERSION
 
 
 opcode_info = namedtuple('opcode_info', ['argsize'])
@@ -35,128 +34,6 @@ def get_code_object(obj):
     return getattr(obj, '__code__', getattr(obj, 'func_code', None))
 
 
-def _make_bytecode_table():
-    # Note some opcodes are supported here for analysis but not later
-    # in the compilation pipeline.
-    version_specific = []
-
-    if sys.version_info[0] == 2:
-        version_specific += [
-            ('BINARY_DIVIDE', 0),
-            ('DELETE_SLICE+0', 0),
-            ('DELETE_SLICE+1', 0),
-            ('DELETE_SLICE+2', 0),
-            ('DELETE_SLICE+3', 0),
-            ('DUP_TOPX', 2),
-            ('INPLACE_DIVIDE', 0),
-            ('PRINT_ITEM', 0),
-            ('PRINT_NEWLINE', 0),
-            ('ROT_FOUR', 0),
-            ('SLICE+0', 0),
-            ('SLICE+1', 0),
-            ('SLICE+2', 0),
-            ('SLICE+3', 0),
-            ('STORE_SLICE+0', 0),
-            ('STORE_SLICE+1', 0),
-            ('STORE_SLICE+2', 0),
-            ('STORE_SLICE+3', 0),
-        ]
-
-    if sys.version_info[0] == 3:
-        version_specific += [
-            ('DUP_TOP_TWO', 0)
-        ]
-
-    if sys.version_info[:2] <= (3, 4):
-        version_specific += [
-            ('STORE_MAP', 0),
-        ]
-
-    if sys.version_info[:2] >= (3, 5):   # python 3.5+
-        version_specific += [
-            ('BINARY_MATRIX_MULTIPLY', 0),
-            ('INPLACE_MATRIX_MULTIPLY', 0),
-        ]
-
-    bytecodes = [
-        # opname, operandlen
-        ('BINARY_ADD', 0),
-        ('BINARY_TRUE_DIVIDE', 0),
-        ('BINARY_MULTIPLY', 0),
-        ('BINARY_SUBSCR', 0),
-        ('BINARY_SUBTRACT', 0),
-        ('BINARY_FLOOR_DIVIDE', 0),
-        ('BINARY_MODULO', 0),
-        ('BINARY_POWER', 0),
-        ('BINARY_AND', 0),
-        ('BINARY_OR', 0),
-        ('BINARY_XOR', 0),
-        ('BINARY_LSHIFT', 0),
-        ('BINARY_RSHIFT', 0),
-        ('BREAK_LOOP', 0),
-        ('BUILD_LIST', 2),
-        ('BUILD_MAP', 2),
-        ('BUILD_SET', 2),
-        ('BUILD_SLICE', 2),
-        ('BUILD_TUPLE', 2),
-        ('CALL_FUNCTION', 2),
-        ('CALL_FUNCTION_VAR', 2),
-        ('COMPARE_OP', 2),
-        ('DELETE_ATTR', 2),
-        ('DELETE_SUBSCR', 0),
-        ('DUP_TOP', 0),
-        ('EXTENDED_ARG', 2),
-        ('FOR_ITER', 2),
-        ('GET_ITER', 0),
-        ('INPLACE_ADD', 0),
-        ('INPLACE_SUBTRACT', 0),
-        ('INPLACE_MULTIPLY', 0),
-        ('INPLACE_TRUE_DIVIDE', 0),
-        ('INPLACE_FLOOR_DIVIDE', 0),
-        ('INPLACE_MODULO', 0),
-        ('INPLACE_POWER', 0),
-        ('INPLACE_AND', 0),
-        ('INPLACE_OR', 0),
-        ('INPLACE_XOR', 0),
-        ('INPLACE_LSHIFT', 0),
-        ('INPLACE_RSHIFT', 0),
-        ('JUMP_ABSOLUTE', 2),
-        ('JUMP_FORWARD', 2),
-        ('JUMP_IF_TRUE_OR_POP', 2),
-        ('JUMP_IF_FALSE_OR_POP', 2),
-        ('LOAD_ATTR', 2),
-        ('LOAD_CLOSURE', 2),
-        ('LOAD_CONST', 2),
-        ('LOAD_FAST', 2),
-        ('LOAD_GLOBAL', 2),
-        ('LOAD_DEREF', 2),
-        ('MAKE_CLOSURE', 2),
-        ('MAKE_FUNCTION', 2),
-        ('POP_BLOCK', 0),
-        ('POP_JUMP_IF_FALSE', 2),
-        ('POP_JUMP_IF_TRUE', 2),
-        ('POP_TOP', 0),
-        ('RAISE_VARARGS', 2),
-        ('RETURN_VALUE', 0),
-        ('ROT_THREE', 0),
-        ('ROT_TWO', 0),
-        ('SETUP_LOOP', 2),
-        ('STORE_ATTR', 2),
-        ('STORE_DEREF', 2),
-        ('STORE_FAST', 2),
-        ('STORE_SUBSCR', 0),
-        ('UNARY_POSITIVE', 0),
-        ('UNARY_NEGATIVE', 0),
-        ('UNARY_INVERT', 0),
-        ('UNARY_NOT', 0),
-        ('UNPACK_SEQUENCE', 2),
-        ('YIELD_VALUE', 0),
-    ] + version_specific
-
-    return dict((dis.opmap[opname], opcode_info(argsize=argsize))
-                for opname, argsize in bytecodes)
-
-
 def _as_opcodes(seq):
     lst = []
     for s in seq:
@@ -166,13 +43,12 @@ def _as_opcodes(seq):
     return lst
 
 
-BYTECODE_TABLE = _make_bytecode_table()
-
 JREL_OPS = frozenset(dis.hasjrel)
 JABS_OPS = frozenset(dis.hasjabs)
 JUMP_OPS = JREL_OPS | JABS_OPS
 TERM_OPS = frozenset(_as_opcodes(['RETURN_VALUE', 'RAISE_VARARGS']))
 EXTENDED_ARG = dis.EXTENDED_ARG
+HAVE_ARGUMENT = dis.HAVE_ARGUMENT
 
 
 class ByteCodeInst(object):
@@ -190,18 +66,14 @@ class ByteCodeInst(object):
     '''
     __slots__ = 'offset', 'next', 'opcode', 'opname', 'arg', 'lineno'
 
-    def __init__(self, offset, opcode, arg):
+    def __init__(self, offset, opcode, arg, nextoffset):
         self.offset = offset
-        self.next = offset + BYTECODE_TABLE[opcode].argsize + 1
+        self.next = nextoffset
         self.opcode = opcode
         self.opname = dis.opname[opcode]
         self.arg = arg
         self.lineno = -1  # unknown line number
 
-    @classmethod
-    def get(cls, offset, opname, arg):
-        return cls(offset, dis.opmap[opname], arg)
-
     @property
     def is_jump(self):
         return self.opcode in JUMP_OPS
@@ -234,38 +106,63 @@ class ByteCodeInst(object):
             return 0
 
 
+if sys.version_info[:2] >= (3, 6):
+    CODE_LEN = 1
+    ARG_LEN = 1
+    NO_ARG_LEN = 1
+else:
+    CODE_LEN = 1
+    ARG_LEN = 2
+    NO_ARG_LEN = 0
+
+
+# Adapted from Lib/dis.py
+def _unpack_opargs(code):
+    """
+    Returns a 4-int-tuple of
+    (bytecode offset, opcode, argument, offset of next bytecode).
+    """
+    if sys.version_info[0] < 3:
+        code = list(map(ord, code))
+
+    extended_arg = 0
+    n = len(code)
+    offset = i = 0
+    while i < n:
+        op = code[i]
+        i += CODE_LEN
+        if op >= HAVE_ARGUMENT:
+            arg = code[i] | extended_arg
+            for j in range(ARG_LEN):
+                arg |= code[i + j] << (8 * j)
+            i += ARG_LEN
+            if op == EXTENDED_ARG:
+                extended_arg = arg << 8 * ARG_LEN
+                continue
+        else:
+            arg = None
+            i += NO_ARG_LEN
+
+        extended_arg = 0
+        yield (offset, op, arg, i)
+        offset = i  # Mark inst offset at first extended
+
+
 class ByteCodeIter(object):
     def __init__(self, code):
         self.code = code
-        if PYVERSION > (3, 0):
-            self.iter = enumerate(self.code.co_code)
-        else:
-            self.iter = ((i, ord(x)) for i, x in enumerate(self.code.co_code))
+        self.iter = iter(_unpack_opargs(self.code.co_code))
 
     def __iter__(self):
         return self
 
     def _fetch_opcode(self):
-        offset, opcode = next(self.iter)
-        try:
-            info = BYTECODE_TABLE[opcode]
-        except KeyError:
-            ts = "offset=%d opcode=0x%x opname=%s"
-            tv = offset, opcode, dis.opname[opcode]
-            raise NotImplementedError(ts % tv)
-        if info.argsize:
-            arg = self.read_arg(info.argsize)
-        else:
-            arg = None
-        return offset, opcode, arg
+        return next(self.iter)
 
     def next(self):
-        offset, opcode, arg = self._fetch_opcode()
-        if opcode == EXTENDED_ARG:
-            hi_arg = arg
-            offset, opcode, lo_arg = self._fetch_opcode()
-            arg = (hi_arg << 16) + lo_arg
-        return offset, ByteCodeInst(offset=offset, opcode=opcode, arg=arg)
+        offset, opcode, arg, nextoffset = self._fetch_opcode()
+        return offset, ByteCodeInst(offset=offset, opcode=opcode, arg=arg,
+                                    nextoffset=nextoffset)
 
     __next__ = next
 
@@ -282,7 +179,7 @@ class ByteCode(object):
     The decoded bytecode of a function, and related information.
     """
     __slots__ = ('func_id', 'co_names', 'co_varnames', 'co_consts',
-                 'co_freevars', 'table', 'labels')
+                 'co_cellvars', 'co_freevars', 'table', 'labels')
 
     def __init__(self, func_id):
         code = func_id.code
@@ -298,6 +195,7 @@ class ByteCode(object):
         self.co_names = code.co_names
         self.co_varnames = code.co_varnames
         self.co_consts = code.co_consts
+        self.co_cellvars = code.co_cellvars
         self.co_freevars = code.co_freevars
         self.table = table
         self.labels = sorted(labels)
@@ -403,7 +301,7 @@ class FunctionIdentity(object):
             func_qualname = func.__name__
 
         self = cls()
-        self.func = pyfunc
+        self.func = func
         self.func_qualname = func_qualname
         self.func_name = func_qualname.split('.')[-1]
         self.code = code
