@@ -1,5 +1,6 @@
 from __future__ import print_function, division, absolute_import
 
+import os
 import inspect
 from contextlib import contextmanager
 from collections import namedtuple, defaultdict
@@ -7,6 +8,7 @@ from pprint import pprint
 import sys
 import warnings
 import traceback
+import threading
 from .tracing import trace, event
 
 from numba import (bytecode, interpreter, funcdesc, postproc,
@@ -14,6 +16,12 @@ from numba import (bytecode, interpreter, funcdesc, postproc,
                    errors, types, ir, types, rewrites, transforms)
 from numba.targets import cpu, callconv
 from numba.annotations import type_annotations
+from numba.parfor import PreParforPass, ParforPass, Parfor
+from numba.inline_closurecall import InlineClosureCallPass
+
+
+# Lock for the preventing multiple compiler execution
+lock_compiler = threading.RLock()
 
 
 class Flags(utils.ConfigOptions):
@@ -32,12 +40,17 @@ class Flags(utils.ConfigOptions):
         # Release GIL inside the native function
         'release_gil': False,
         'no_compile': False,
+        'debuginfo': False,
         'boundcheck': False,
         'forceinline': False,
         'no_cpython_wrapper': False,
+        # Enable automatic parallel optimization, can be fine-tuned by taking
+        # a dictionary of sub-options instead of a boolean, see parfor.py for detail.
+        'auto_parallel': cpu.ParallelOptions(False),
         'nrt': False,
         'no_rewrites': False,
         'error_model': 'python',
+        'fastmath': False,
     }
 
 
@@ -267,6 +280,7 @@ class Pipeline(object):
         self.bc = None
         self.func_id = None
         self.func_ir = None
+        self.func_ir_original = None  # used for fallback
         self.lifted = None
         self.lifted_from = None
         self.typemap = None
@@ -377,6 +391,9 @@ class Pipeline(object):
     def stage_process_ir(self):
         ir_processing_stage(self.func_ir)
 
+    def stage_preserve_ir(self):
+        self.func_ir_original = self.func_ir.copy()
+
     def frontend_looplift(self):
         """
         Loop lifting analysis and transformation
@@ -410,6 +427,7 @@ class Pipeline(object):
         """
         Front-end: Analyze bytecode, generate Numba IR, infer types
         """
+        self.func_ir = self.func_ir_original or self.func_ir
         if self.flags.enable_looplift:
             assert not self.lifted
             cres = self.frontend_looplift()
@@ -467,6 +485,69 @@ class Pipeline(object):
             rewrites.rewrite_registry.apply('after-inference',
                                             self, self.func_ir)
 
+    def stage_pre_parfor_pass(self):
+        """
+        Preprocessing for data-parallel computations.
+        """
+        # Ensure we have an IR and type information.
+        assert self.func_ir
+        preparfor_pass = PreParforPass(self.func_ir, self.type_annotation.typemap,
+            self.type_annotation.calltypes, self.typingctx,
+            self.flags.auto_parallel)
+        preparfor_pass.run()
+
+    def stage_parfor_pass(self):
+        """
+        Convert data-parallel computations into Parfor nodes
+        """
+        # Ensure we have an IR and type information.
+        assert self.func_ir
+        parfor_pass = ParforPass(self.func_ir, self.type_annotation.typemap,
+            self.type_annotation.calltypes, self.return_type, self.typingctx,
+            self.flags.auto_parallel)
+        parfor_pass.run()
+
+        if config.WARNINGS:
+            # check the parfor pass worked and warn if it didn't
+            has_parfor = False
+            for blk in self.func_ir.blocks.values():
+                for stmnt in blk.body:
+                    if isinstance(stmnt, Parfor):
+                        has_parfor = True
+                        break
+                else:
+                    continue
+                break
+
+            if not has_parfor:
+                # parfor calls the compiler chain again with a string
+                if not self.func_ir.loc.filename == '<string>':
+                    msg = ("parallel=True was specified but no transformation"
+                           " for parallel execution was possible.")
+                    warnings.warn_explicit(msg,
+                                errors.NumbaWarning,
+                                self.func_id.filename,
+                                self.func_id.firstlineno)
+
+
+    def stage_inline_pass(self):
+        """
+        Inline calls to locally defined closures.
+        """
+        # Ensure we have an IR and type information.
+        assert self.func_ir
+        inline_pass = InlineClosureCallPass(self.func_ir, self.flags.auto_parallel)
+        inline_pass.run()
+        # Remove all Dels, and re-run postproc
+        post_proc = postproc.PostProcessor(self.func_ir)
+        post_proc.run()
+
+        if config.DEBUG or config.DUMP_IR:
+            name = self.func_ir.func_id.func_qualname
+            print(("IR DUMP: %s" % name).center(80, "-"))
+            self.func_ir.dump()
+
+
     def stage_annotate_type(self):
         """
         Create type annotation after type inference
@@ -486,7 +567,8 @@ class Pipeline(object):
             print(self.type_annotation)
             print('=' * 80)
         if config.HTML:
-            self.type_annotation.html_annotate()
+            with open(config.HTML, 'w') as fout:
+                self.type_annotation.html_annotate(fout)
 
     def backend_object_mode(self):
         """
@@ -609,11 +691,18 @@ class Pipeline(object):
                 pm.add_stage(self.stage_analyze_bytecode, "analyzing bytecode")
             pm.add_stage(self.stage_process_ir, "processing IR")
             if not self.flags.no_rewrites:
+                if self.status.can_fallback:
+                    pm.add_stage(self.stage_preserve_ir, "preserve IR for fallback")
                 pm.add_stage(self.stage_generic_rewrites, "nopython rewrites")
+            pm.add_stage(self.stage_inline_pass, "inline calls to locally defined closures")
             pm.add_stage(self.stage_nopython_frontend, "nopython frontend")
             pm.add_stage(self.stage_annotate_type, "annotate type")
+            if self.flags.auto_parallel.enabled:
+                pm.add_stage(self.stage_pre_parfor_pass, "Preprocessing for parfors")
             if not self.flags.no_rewrites:
                 pm.add_stage(self.stage_nopython_rewrites, "nopython rewrites")
+            if self.flags.auto_parallel.enabled:
+                pm.add_stage(self.stage_parfor_pass, "convert to parfors")
             pm.add_stage(self.stage_nopython_backend, "nopython mode backend")
             pm.add_stage(self.stage_cleanup, "cleanup intermediate results")
 
@@ -661,10 +750,16 @@ def _make_subtarget(targetctx, flags):
     Make a new target context from the given target context and flags.
     """
     subtargetoptions = {}
+    if flags.debuginfo:
+        subtargetoptions['enable_debuginfo'] = True
     if flags.boundcheck:
         subtargetoptions['enable_boundcheck'] = True
     if flags.nrt:
         subtargetoptions['enable_nrt'] = True
+    if flags.auto_parallel:
+        subtargetoptions['auto_parallel'] = flags.auto_parallel
+    if flags.fastmath:
+        subtargetoptions['enable_fastmath'] = True
     error_model = callconv.create_error_model(flags.error_model, targetctx)
     subtargetoptions['error_model'] = error_model
 
@@ -739,7 +834,8 @@ def legalize_return_type(return_type, interp, targetctx):
 
     elif (isinstance(return_type, types.Function) or
             isinstance(return_type, types.Phantom)):
-        raise TypeError("Can't return function object in nopython mode")
+        msg = "Can't return function object ({}) in nopython mode"
+        raise TypeError(msg.format(return_type))
 
 
 def translate_stage(func_id, bytecode):
