@@ -15,6 +15,7 @@ import numpy as np
 from . import driver as _driver
 from . import devices
 from numba import dummyarray, types, numpy_support
+from numba.unsafe.ndarray import to_fixed_tuple
 
 try:
     long
@@ -97,13 +98,15 @@ class DeviceNDArrayBase(object):
             else:
                 self.alloc_size = _driver.device_memory_size(gpu_data)
         else:
-            gpu_data = None
+            # Make NULL pointer for empty allocation
+            gpu_data = _driver.MemoryPointer(context=devices.get_context(),
+                                             pointer=c_void_p(0), size=0)
             self.alloc_size = 0
 
         self.gpu_data = gpu_data
 
         self.__writeback = writeback    # should deprecate the use of this
-        self.stream = 0
+        self.stream = stream
 
     def bind(self, stream=0):
         """Bind a CUDA stream to this object so that all subsequent operation
@@ -113,6 +116,21 @@ class DeviceNDArrayBase(object):
         clone.stream = stream
         return clone
 
+    @property
+    def T(self):
+        return self.transpose()
+
+    def transpose(self, axes=None):
+        if axes and tuple(axes) == tuple(range(self.ndim)):
+            return self
+        elif self.ndim != 2:
+            raise NotImplementedError("transposing a non-2D DeviceNDArray isn't supported")
+        elif axes is not None and set(axes) != set(range(self.ndim)):
+            raise ValueError("invalid axes list %r" % (axes,))
+        else:
+            from numba.cuda.kernels.transpose import transpose
+            return transpose(self)
+
     def _default_stream(self, stream):
         return self.stream if not stream else stream
 
@@ -134,6 +152,7 @@ class DeviceNDArrayBase(object):
         else:
             return self.gpu_data.device_ctypes_pointer
 
+    @devices.require_context
     def copy_to_device(self, ary, stream=0):
         """Copy `ary` to `self`.
 
@@ -143,14 +162,34 @@ class DeviceNDArrayBase(object):
         if ary.size == 0:
             # Nothing to do
             return
+
+        sentry_contiguous(self)
         stream = self._default_stream(stream)
+
         if _driver.is_device_memory(ary):
+            sentry_contiguous(ary)
+
+            if self.flags['C_CONTIGUOUS'] != ary.flags['C_CONTIGUOUS']:
+                raise ValueError("Can't copy %s-contiguous array to a %s-contiguous array" % (
+                    'C' if ary.flags['C_CONTIGUOUS'] else 'F',
+                    'C' if self.flags['C_CONTIGUOUS'] else 'F',
+                ))
+
             sz = min(self.alloc_size, ary.alloc_size)
             _driver.device_to_device(self, ary, sz, stream=stream)
         else:
+            # Ensure same contiguous-nous. Only copies (host-side)
+            # if necessary (e.g. it needs to materialize a strided view)
+            ary = np.array(
+                ary,
+                order='C' if self.flags['C_CONTIGUOUS'] else 'F',
+                subok=True,
+                copy=False)
+
             sz = min(_driver.host_memory_size(ary), self.alloc_size)
             _driver.host_to_device(self, ary, sz, stream=stream)
 
+    @devices.require_context
     def copy_to_host(self, ary=None, stream=0):
         """Copy ``self`` to ``ary`` or create a new Numpy ndarray
         if ``ary`` is ``None``.
@@ -240,6 +279,17 @@ class DeviceNDArrayBase(object):
         """
         return self.gpu_data
 
+    def get_ipc_handle(self):
+        """
+        Returns a *IpcArrayHandle* object that is safe to serialize and transfer
+        to another process to share the local allocation.
+
+        Note: this feature is only available on Linux.
+        """
+        ipch = devices.get_context().get_ipc_handle(self.gpu_data)
+        desc = dict(shape=self.shape, strides=self.strides, dtype=self.dtype)
+        return IpcArrayHandle(ipc_handle=ipch, array_desc=desc)
+
 
 class DeviceRecord(DeviceNDArrayBase):
     '''
@@ -251,6 +301,16 @@ class DeviceRecord(DeviceNDArrayBase):
         super(DeviceRecord, self).__init__(shape, strides, dtype, stream,
                                            gpu_data)
 
+    @property
+    def flags(self):
+        """
+        For `numpy.ndarray` compatibility. Ideally this would return a
+        `np.core.multiarray.flagsobj`, but that needs to be constructed
+        with an existing `numpy.ndarray` (as the C- and F- contiguous flags
+        aren't writeable).
+        """
+        return dict(self._dummy.flags) # defensive copy
+
     @property
     def _numba_type_(self):
         """
@@ -270,12 +330,28 @@ class DeviceNDArray(DeviceNDArrayBase):
         '''
         return self._dummy.is_f_contig
 
+    @property
+    def flags(self):
+        """
+        For `numpy.ndarray` compatibility. Ideally this would return a
+        `np.core.multiarray.flagsobj`, but that needs to be constructed
+        with an existing `numpy.ndarray` (as the C- and F- contiguous flags
+        aren't writeable).
+        """
+        return dict(self._dummy.flags) # defensive copy
+
     def is_c_contiguous(self):
         '''
         Return true if the array is C-contiguous.
         '''
         return self._dummy.is_c_contig
 
+    def __array__(self, dtype=None):
+        """
+        :return: an `numpy.ndarray`, so copies to the host.
+        """
+        return self.copy_to_host().__array__(dtype)
+
     def reshape(self, *newshape, **kws):
         """
         Reshape the array without changing its contents, similarly to
@@ -317,6 +393,7 @@ class DeviceNDArray(DeviceNDArrayBase):
         else:
             raise NotImplementedError("operation requires copying")
 
+    @devices.require_context
     def __getitem__(self, item):
         return self._do_getitem(item)
 
@@ -349,6 +426,135 @@ class DeviceNDArray(DeviceNDArrayBase):
             return cls(shape=arr.shape, strides=arr.strides,
                        dtype=self.dtype, gpu_data=newdata, stream=stream)
 
+    @devices.require_context
+    def __setitem__(self, key, value):
+        return self._do_setitem(key, value)
+
+    def setitem(self, key, value, stream=0):
+        """Do `__setitem__(key, value)` with CUDA stream
+        """
+        return self._so_getitem(key, value, stream)
+
+    def _do_setitem(self, key, value, stream=0):
+
+        stream = self._default_stream(stream)
+
+        # (1) prepare LHS
+
+        arr = self._dummy.__getitem__(key)
+        newdata = self.gpu_data.view(*arr.extent)
+
+        if isinstance(arr, dummyarray.Element):
+            # convert to a 1d array
+            shape = (1,)
+            strides = (self.dtype.itemsize,)
+        else:
+            shape = arr.shape
+            strides = arr.strides
+
+        lhs = type(self)(
+            shape=shape,
+            strides=strides,
+            dtype=self.dtype,
+            gpu_data=newdata,
+            stream=stream)
+
+        # (2) prepare RHS
+
+        rhs, _ = auto_device(value, stream=stream)
+        if rhs.ndim > lhs.ndim:
+            raise ValueError("Can't assign %s-D array to %s-D self" % (
+                rhs.ndim,
+                lhs.ndim))
+        rhs_shape = np.ones(lhs.ndim, dtype=np.int64)
+        rhs_shape[-rhs.ndim:] = rhs.shape
+        rhs = rhs.reshape(*rhs_shape)
+        for i, (l, r) in enumerate(zip(lhs.shape, rhs.shape)):
+            if r != 1 and l != r:
+                raise ValueError("Can't copy sequence with size %d to array axis %d with dimension %d" % (
+                    r,
+                    i,
+                    l))
+
+        # (3) do the copy
+
+        from numba import cuda  # circular!
+
+        n_elements = np.prod(lhs.shape)
+
+        # need to have static array sizes for cuda.local.array, so bake in the
+        # dimensions
+        ndim = lhs.ndim
+
+        @cuda.jit
+        def kernel(lhs, rhs):
+            location = cuda.grid(1)
+
+            if location >= n_elements:
+                # bake n_elements into the kernel, better than passing it in
+                # as another argument.
+                return
+
+            # [0, :] is the to-index (into `lhs`)
+            # [1, :] is the from-index (into `rhs`)
+            idx = cuda.local.array(
+                shape=(2, ndim),
+                dtype=types.int64)
+
+            for i in range(ndim - 1, -1, -1):
+                idx[0, i] = location % lhs.shape[i]
+                idx[1, i] = (location % lhs.shape[i]) * (rhs.shape[i] > 1)
+                location //= lhs.shape[i]
+
+            lhs[to_fixed_tuple(idx[0], ndim)] = rhs[to_fixed_tuple(idx[1], ndim)]
+
+        kernel.forall(n_elements, stream=stream)(lhs, rhs)
+
+
+
+class IpcArrayHandle(object):
+    """
+    An IPC array handle that can be serialized and transfer to another process
+    in the same machine for share a GPU allocation.
+
+    On the destination process, use the *.open()* method to creates a new
+    *DeviceNDArray* object that shares the allocation from the original process.
+    To release the resources, call the *.close()* method.  After that, the
+    destination can no longer use the shared array object.  (Note: the
+    underlying weakref to the resource is now dead.)
+
+    This object implements the context-manager interface that calls the
+    *.open()* and *.close()* method automatically::
+
+        with the_ipc_array_handle as ipc_array:
+            # use ipc_array here as a normal gpu array object
+            some_code(ipc_array)
+        # ipc_array is dead at this point
+    """
+    def __init__(self, ipc_handle, array_desc):
+        self._array_desc = array_desc
+        self._ipc_handle = ipc_handle
+
+    def open(self):
+        """
+        Returns a new *DeviceNDArray* that shares the allocation from the
+        original process.  Must not be used on the original process.
+        """
+        dptr = self._ipc_handle.open(devices.get_context())
+        return DeviceNDArray(gpu_data=dptr, **self._array_desc)
+
+    def close(self):
+        """
+        Closes the IPC handle to the array.
+        """
+        self._ipc_handle.close()
+
+    def __enter__(self):
+        return self.open()
+
+    def __exit__(self, type, value, traceback):
+        self.close()
+
 
 class MappedNDArray(DeviceNDArrayBase, np.ndarray):
     """
@@ -397,10 +603,19 @@ def auto_device(obj, stream=0, copy=True):
     if _driver.is_device_memory(obj):
         return obj, False
     else:
-        sentry_contiguous(obj)
         if isinstance(obj, np.void):
             devobj = from_record_like(obj, stream=stream)
         else:
+            # This allows you to pass non-array objects like constants
+            # and objects implementing the
+            # [array interface](https://docs.scipy.org/doc/numpy-1.13.0/reference/arrays.interface.html)
+            # into this function (with no overhead -- copies -- for `obj`s
+            # that are already `ndarray`s.
+            obj = np.array(
+                obj,
+                copy=False,
+                subok=True)
+            sentry_contiguous(obj)
             devobj = from_array_like(obj, stream=stream)
         if copy:
             devobj.copy_to_device(obj, stream=stream)
