@@ -30,12 +30,14 @@ from numba import utils, mviewbuf
 from .error import CudaSupportError, CudaDriverError
 from .drvapi import API_PROTOTYPES
 from .drvapi import cu_occupancy_b2d_size
-from . import enums, drvapi
-from numba import config
+from . import enums, drvapi, _extras
+from numba import config, serialize
 from numba.utils import longint as long
 
+
 VERBOSE_JIT_LOG = int(os.environ.get('NUMBAPRO_VERBOSE_CU_JIT_LOG', 1))
 MIN_REQUIRED_CC = (2, 0)
+SUPPORTS_IPC = sys.platform.startswith('linux')
 
 
 def _make_logger():
@@ -203,7 +205,10 @@ class Driver(object):
         self.pid = None
         try:
             if config.DISABLE_CUDA:
-                raise CudaSupportError("CUDA disabled by user")
+                msg = ("CUDA is disabled due to setting NUMBA_DISABLE_CUDA=1 "
+                       "in the environment, or because CUDA is unsupported on "
+                       "32-bit systems.")
+                raise CudaSupportError(msg)
             self.lib = find_driver()
         except CudaSupportError as e:
             self.is_initialized = True
@@ -224,6 +229,25 @@ class Driver(object):
         else:
             self.pid = _getpid()
 
+        self._initialize_extras()
+
+    def _initialize_extras(self):
+        # set pointer to original cuIpcOpenMemHandle
+        set_proto = ctypes.CFUNCTYPE(None, c_void_p)
+        set_cuIpcOpenMemHandle = set_proto(_extras.set_cuIpcOpenMemHandle)
+        set_cuIpcOpenMemHandle(self._find_api('cuIpcOpenMemHandle'))
+        # bind caller to cuIpcOpenMemHandle that fixes the ABI
+        call_proto = ctypes.CFUNCTYPE(c_int,
+                                      ctypes.POINTER(drvapi.cu_device_ptr),
+                                      ctypes.POINTER(drvapi.cu_ipc_mem_handle),
+                                      ctypes.c_uint)
+        call_cuIpcOpenMemHandle = call_proto(_extras.call_cuIpcOpenMemHandle)
+        call_cuIpcOpenMemHandle.__name__ = 'call_cuIpcOpenMemHandle'
+        safe_call = self._wrap_api_call('call_cuIpcOpenMemHandle',
+                                        call_cuIpcOpenMemHandle)
+        # override cuIpcOpenMemHandle
+        self.cuIpcOpenMemHandle = safe_call
+
     @property
     def is_available(self):
         if not self.is_initialized:
@@ -252,13 +276,16 @@ class Driver(object):
         libfn.restype = restype
         libfn.argtypes = argtypes
 
+        safe_call = self._wrap_api_call(fname, libfn)
+        setattr(self, fname, safe_call)
+        return safe_call
+
+    def _wrap_api_call(self, fname, libfn):
         @functools.wraps(libfn)
         def safe_cuda_api_call(*args):
             _logger.debug('call driver api: %s', libfn.__name__)
             retcode = libfn(*args)
             self._check_error(fname, retcode)
-
-        setattr(self, fname, safe_cuda_api_call)
         return safe_cuda_api_call
 
     def _find_api(self, fname):
@@ -289,7 +316,7 @@ class Driver(object):
             _logger.error(msg)
             if retcode == enums.CUDA_ERROR_NOT_INITIALIZED:
                 # Detect forking
-                if _getpid() != self.pid:
+                if self.pid is not None and _getpid() != self.pid:
                     msg = 'pid %s forked from pid %s after CUDA driver init'
                     _logger.critical(msg, _getpid(), self.pid)
                     raise CudaDriverError("CUDA initialized before forking")
@@ -328,6 +355,7 @@ class Driver(object):
             return None
         return handle
 
+
 driver = Driver()
 
 
@@ -582,8 +610,7 @@ class Context(object):
     def get_max_potential_block_size(self, func, b2d_func, memsize, blocksizelimit, flags=None):
         """Suggest a launch configuration with reasonable occupancy.
         :param func: kernel for which occupancy is calculated
-        :param b2d_func: function that calculates how much per-block dynamic shared memory 'func'
-          uses based on the block size.
+        :param b2d_func: function that calculates how much per-block dynamic shared memory 'func' uses based on the block size.
         :param memsize: per-block dynamic shared memory usage intended, in bytes
         :param blocksizelimit: maximum block size the kernel is designed to handle"""
 
@@ -726,6 +753,25 @@ class Context(object):
     def memunpin(self, pointer):
         raise NotImplementedError
 
+    def get_ipc_handle(self, memory):
+        """
+        Returns a *IpcHandle* from a GPU allocation.
+        """
+        if not SUPPORTS_IPC:
+            raise OSError('OS does not support CUDA IPC')
+        ipchandle = drvapi.cu_ipc_mem_handle()
+        driver.cuIpcGetMemHandle(ctypes.cast(ipchandle, ctypes.POINTER(drvapi.cu_ipc_mem_handle)), memory.handle)
+        return IpcHandle(memory, ipchandle, memory.size)
+
+    def open_ipc_handle(self, handle, size):
+        # open the IPC handle to get the device pointer
+        dptr = drvapi.cu_device_ptr()
+        flags = 1  # CU_IPC_MEM_LAZY_ENABLE_PEER_ACCESS
+        driver.cuIpcOpenMemHandle(byref(dptr), handle, flags)
+        # wrap it
+        return MemoryPointer(context=weakref.proxy(self), pointer=dptr,
+                             size=size)
+
     def create_module_ptx(self, ptx):
         if isinstance(ptx, str):
             ptx = ptx.encode('utf8')
@@ -868,6 +914,72 @@ def _module_finalizer(context, handle):
     return core
 
 
+class IpcHandle(object):
+    """
+    Internal IPC handle.
+
+    Serialization of the CUDA IPC handle object is implemented here.
+
+    The *base* attribute is a reference to the original allocation to keep it
+    alive.  The *handle* is a ctypes object of the CUDA IPC handle. The *size*
+    is the allocation size.
+    """
+    def __init__(self, base, handle, size):
+        self.base = base
+        self.handle = handle
+        self.size = size
+        # remember if the handle is already opened
+        self._opened_mem = None
+
+    def open(self, context):
+        """
+        Import the IPC memory and returns a raw CUDA memory pointer object
+        """
+        if self.base is not None:
+            raise ValueError('opening IpcHandle from original process')
+
+        if self._opened_mem is not None:
+            raise ValueError('IpcHandle is already opened')
+
+        mem = context.open_ipc_handle(self.handle, self.size)
+        # this object owns the opened allocation
+        # note: it is required the memory be freed after the ipc handle is
+        #       closed by the importing context.
+        self._opened_mem = mem
+        return mem.own()
+
+    def open_array(self, context, shape, dtype, strides=None):
+        """
+        Simliar to `.open()` but returns an device array.
+        """
+        from . import devicearray
+
+        # by default, set strides to itemsize
+        if strides is None:
+            strides = dtype.itemsize
+        dptr = self.open(context)
+        # read the device pointer as an array
+        return devicearray.DeviceNDArray(shape=shape, strides=strides,
+                                         dtype=dtype, gpu_data=dptr)
+
+    def close(self):
+        if self._opened_mem is None:
+            raise ValueError('IpcHandle not opened')
+        driver.cuIpcCloseMemHandle(self._opened_mem.handle)
+        self._opened_mem = None
+
+    def __reduce__(self):
+        # Preprocess the IPC handle, which is defined as a byte array.
+        preprocessed_handle = tuple(self.handle)
+        args = (self.__class__, preprocessed_handle, self.size)
+        return (serialize._rebuild_reduction, args)
+
+    @classmethod
+    def _rebuild(cls, handle_ary, size):
+        handle = drvapi.cu_ipc_mem_handle(*handle_ary)
+        return cls(base=None, handle=handle, size=size)
+
+
 class MemoryPointer(object):
     __cuda_memory__ = True
 
@@ -910,14 +1022,23 @@ class MemoryPointer(object):
             driver.cuMemsetD8(self.device_pointer, byte, count)
 
     def view(self, start, stop=None):
-        base = self.device_pointer.value + start
         if stop is None:
             size = self.size - start
         else:
             size = stop - start
-        assert size > 0, "zero or negative memory size"
-        pointer = drvapi.cu_device_ptr(base)
-        view = MemoryPointer(self.context, pointer, size, owner=self.owner)
+
+        # Handle NULL/empty memory buffer
+        if self.device_pointer.value is None:
+            if size != 0:
+                raise RuntimeError("non-empty slice into empty slice")
+            view = self      # new view is just a reference to self
+        # Handle normal case
+        else:
+            base = self.device_pointer.value + start
+            if size < 0:
+                raise RuntimeError('size cannot be negative')
+            pointer = drvapi.cu_device_ptr(base)
+            view = MemoryPointer(self.context, pointer, size, owner=self.owner)
         return OwnedPointer(weakref.proxy(self.owner), view)
 
     @property
@@ -980,10 +1101,14 @@ class OwnedPointer(object):
         mem = self._mem
 
         def deref():
-            mem.refct -= 1
-            assert mem.refct >= 0
-            if mem.refct == 0:
-                mem.free()
+            try:
+                mem.refct -= 1
+                assert mem.refct >= 0
+                if mem.refct == 0:
+                    mem.free()
+            except ReferenceError:
+                # ignore reference error here
+                pass
 
         self._mem.refct += 1
         utils.finalize(self, deref)
