@@ -244,55 +244,52 @@ def ptx_threadfence_device(context, builder, sig, args):
     return context.get_dummy_value()
 
 
-@lower(stubs.atomic.add, types.Array, types.intp, types.Any)
-def ptx_atomic_add_intp(context, builder, sig, args):
-    aryty, indty, valty = sig.args
-    ary, ind, val = args
-    dtype = aryty.dtype
-
-    if dtype != valty:
-        raise TypeError("expect %s but got %s" % (dtype, valty))
-    if aryty.ndim != 1:
-        raise TypeError("indexing %d-D array with 1-D index" % (aryty.ndim,))
-
-    lary = context.make_array(aryty)(context, builder, ary)
-    ptr = cgutils.get_item_pointer(builder, aryty, lary, [ind])
-
-    if aryty.dtype == types.float32:
-        lmod = builder.module
-        return builder.call(nvvmutils.declare_atomic_add_float32(lmod), (ptr, val))
-    elif aryty.dtype == types.float64:
-        lmod = builder.module
-        return builder.call(nvvmutils.declare_atomic_add_float64(lmod), (ptr, val))
+def _normalize_indices(context, builder, indty, inds):
+    """
+    Convert integer indices into tuple of intp
+    """
+    if indty in types.integer_domain:
+        indty = types.UniTuple(dtype=indty, count=1)
+        indices = [inds]
     else:
-        return builder.atomic_rmw('add', ptr, val, 'monotonic')
-
-
-@lower(stubs.atomic.add, types.Array, types.UniTuple, types.Any)
-@lower(stubs.atomic.add, types.Array, types.Tuple, types.Any)
-def ptx_atomic_add_tuple(context, builder, sig, args):
-    aryty, indty, valty = sig.args
-    ary, inds, val = args
-    dtype = aryty.dtype
-
-    indices = cgutils.unpack_tuple(builder, inds, count=len(indty))
+        indices = cgutils.unpack_tuple(builder, inds, count=len(indty))
     indices = [context.cast(builder, i, t, types.intp)
                for t, i in zip(indty, indices)]
+    return indty, indices
 
-    if dtype != valty:
-        raise TypeError("expect %s but got %s" % (dtype, valty))
 
-    if aryty.ndim != len(indty):
-        raise TypeError("indexing %d-D array with %d-D index" %
-                        (aryty.ndim, len(indty)))
+def _atomic_dispatcher(dispatch_fn):
+    def imp(context, builder, sig, args):
+        # The common argument handling code
+        aryty, indty, valty = sig.args
+        ary, inds, val = args
+        dtype = aryty.dtype
 
-    lary = context.make_array(aryty)(context, builder, ary)
-    ptr = cgutils.get_item_pointer(builder, aryty, lary, indices)
+        indty, indices = _normalize_indices(context, builder, indty, inds)
 
-    if aryty.dtype == types.float32:
+        if dtype != valty:
+            raise TypeError("expect %s but got %s" % (dtype, valty))
+
+        if aryty.ndim != len(indty):
+            raise TypeError("indexing %d-D array with %d-D index" %
+                            (aryty.ndim, len(indty)))
+
+        lary = context.make_array(aryty)(context, builder, ary)
+        ptr = cgutils.get_item_pointer(builder, aryty, lary, indices)
+        # dispatcher to implementation base on dtype
+        return dispatch_fn(context, builder, dtype, ptr, val)
+    return imp
+
+
+@lower(stubs.atomic.add, types.Array, types.intp, types.Any)
+@lower(stubs.atomic.add, types.Array, types.UniTuple, types.Any)
+@lower(stubs.atomic.add, types.Array, types.Tuple, types.Any)
+@_atomic_dispatcher
+def ptx_atomic_add_tuple(context, builder, dtype, ptr, val):
+    if dtype == types.float32:
         lmod = builder.module
         return builder.call(nvvmutils.declare_atomic_add_float32(lmod), (ptr, val))
-    elif aryty.dtype == types.float64:
+    elif dtype == types.float64:
         lmod = builder.module
         return builder.call(nvvmutils.declare_atomic_add_float64(lmod), (ptr, val))
     else:
@@ -300,54 +297,56 @@ def ptx_atomic_add_tuple(context, builder, sig, args):
 
 
 @lower(stubs.atomic.max, types.Array, types.intp, types.Any)
-def ptx_atomic_max_intp(context, builder, sig, args):
-    aryty, indty, valty = sig.args
-    ary, ind, val = args
-    dtype = aryty.dtype
-
-    if dtype != valty:
-        raise TypeError("expect %s but got %s" % (dtype, valty))
-    if aryty.ndim != 1:
-        raise TypeError("indexing %d-D array with 1-D index" % (aryty.ndim,))
-
-    lary = context.make_array(aryty)(context, builder, ary)
-    ptr = cgutils.get_item_pointer(builder, aryty, lary, [ind])
-
-    if dtype == types.float64:
-        lmod = builder.module
-        return builder.call(nvvmutils.declare_atomic_max_float64(lmod), (ptr, val))
-    else:
-        raise TypeError('Unimplemented atomic max with %s array' % dtype)
-
-
 @lower(stubs.atomic.max, types.Array, types.Tuple, types.Any)
 @lower(stubs.atomic.max, types.Array, types.UniTuple, types.Any)
-def ptx_atomic_max_tuple(context, builder, sig, args):
-    aryty, indty, valty = sig.args
-    ary, inds, val = args
-    dtype = aryty.dtype
-
-    indices = cgutils.unpack_tuple(builder, inds, count=len(indty))
-    indices = [context.cast(builder, i, t, types.intp)
-               for t, i in zip(indty, indices)]
-
-    if dtype != valty:
-        raise TypeError("expect %s but got %s" % (dtype, valty))
-
-    if aryty.ndim != len(indty):
-        raise TypeError("indexing %d-D array with %d-D index" %
-                        (aryty.ndim, len(indty)))
-
-    lary = context.make_array(aryty)(context, builder, ary)
-    ptr = cgutils.get_item_pointer(builder, aryty, lary, indices)
-
-    if aryty.dtype == types.float64:
-        lmod = builder.module
+@_atomic_dispatcher
+def ptx_atomic_max(context, builder, dtype, ptr, val):
+    lmod = builder.module
+    if dtype == types.float64:
         return builder.call(nvvmutils.declare_atomic_max_float64(lmod), (ptr, val))
+    elif dtype == types.float32:
+        return builder.call(nvvmutils.declare_atomic_max_float32(lmod), (ptr, val))
+    elif dtype in (types.int32, types.int64):
+        return builder.atomic_rmw('max', ptr, val, ordering='monotonic')
+    elif dtype in (types.uint32, types.uint64):
+        return builder.atomic_rmw('umax', ptr, val, ordering='monotonic')
     else:
         raise TypeError('Unimplemented atomic max with %s array' % dtype)
 
 
+@lower(stubs.atomic.min, types.Array, types.intp, types.Any)
+@lower(stubs.atomic.min, types.Array, types.Tuple, types.Any)
+@lower(stubs.atomic.min, types.Array, types.UniTuple, types.Any)
+@_atomic_dispatcher
+def ptx_atomic_min(context, builder, dtype, ptr, val):
+    lmod = builder.module
+    if dtype == types.float64:
+        return builder.call(nvvmutils.declare_atomic_min_float64(lmod), (ptr, val))
+    elif dtype == types.float32:
+        return builder.call(nvvmutils.declare_atomic_min_float32(lmod), (ptr, val))
+    elif dtype in (types.int32, types.int64):
+        return builder.atomic_rmw('min', ptr, val, ordering='monotonic')
+    elif dtype in (types.uint32, types.uint64):
+        return builder.atomic_rmw('umin', ptr, val, ordering='monotonic')
+    else:
+        raise TypeError('Unimplemented atomic min with %s array' % dtype)
+
+
+@lower(stubs.atomic.compare_and_swap, types.Array, types.Any, types.Any)
+def ptx_atomic_cas_tuple(context, builder, sig, args):
+    aryty, oldty, valty = sig.args
+    ary, old, val = args
+    dtype = aryty.dtype
+
+    lary = context.make_array(aryty)(context, builder, ary)
+    zero = context.get_constant(types.intp, 0)
+    ptr = cgutils.get_item_pointer(builder, aryty, lary, (zero,))
+    if aryty.dtype == types.int32:
+        lmod = builder.module
+        return builder.call(nvvmutils.declare_atomic_cas_int32(lmod), (ptr, old, val))
+    else:
+        raise TypeError('Unimplemented atomic compare_and_swap with %s array' % dtype)
+
 
 # -----------------------------------------------------------------------------
 
@@ -372,6 +371,8 @@ def _generic_array(context, builder, shape, dtype, symbol_name, addrspace,
 
         # Create global variable in the requested address-space
         gvmem = lmod.add_global_variable(laryty, symbol_name, addrspace)
+        # Specify alignment to avoid misalignment bug
+        gvmem.align = context.get_abi_sizeof(lldtype)
 
         if elemcount <= 0:
             if can_dynsized:    # dynamic shared memory
