@@ -6,10 +6,10 @@ import os
 import sys
 
 from numba import config, unittest_support as unittest
-from numba.tests.support import captured_stdout
+from numba.tests.support import captured_stdout, SerialMixin
 
 
-class CUDATestCase(unittest.TestCase):
+class CUDATestCase(SerialMixin, unittest.TestCase):
     def tearDown(self):
         from numba.cuda.cudadrv.devices import reset
 
@@ -61,6 +61,9 @@ def captured_cuda_stdout():
     Return a minimal stream-like object capturing the text output of
     either CUDA or the simulator.
     """
+    # Prevent accidentally capturing previously output text
+    sys.stdout.flush()
+
     if config.ENABLE_CUDASIM:
         # The simulator calls print() on Python stdout
         with captured_stdout() as stream:
