@@ -3,7 +3,7 @@ from __future__ import print_function, absolute_import
 from ctypes import c_int, sizeof
 from numba.cuda.cudadrv.driver import host_to_device, device_to_host
 from numba.cuda.cudadrv import devices
-from numba.cuda.testing import unittest
+from numba.cuda.testing import unittest, SerialMixin
 from numba.cuda.testing import skip_on_cudasim
 
 ptx1 = '''
@@ -60,7 +60,7 @@ ptx2 = '''
 
 
 @skip_on_cudasim('CUDA Driver API unsupported in the simulator')
-class TestCudaDriver(unittest.TestCase):
+class TestCudaDriver(SerialMixin, unittest.TestCase):
     def setUp(self):
         self.assertTrue(len(devices.gpus) > 0)
         self.context = devices.get_context()
@@ -113,6 +113,18 @@ class TestCudaDriver(unittest.TestCase):
         for i, v in enumerate(array):
             self.assertEqual(i, v)
 
+    def test_cuda_driver_occupancy(self):
+        module = self.context.create_module_ptx(self.ptx)
+        function = module.get_function('_Z10helloworldPi')
+
+        value = self.context.get_active_blocks_per_multiprocessor(function, 128, 128)
+        print('active blocks:', value)
+        self.assertTrue(value > 0)
+        def b2d(bs): return bs
+        grid, block = self.context.get_max_potential_block_size(function, b2d, 128, 128)
+        print('grid size:', grid, ', block size:', block)
+        self.assertTrue(grid > 0)
+        self.assertTrue(block > 0)
 
 if __name__ == '__main__':
     unittest.main()
