@@ -5,7 +5,7 @@ import sys
 import numpy as np
 from numba import cuda, numpy_support, types
 from numba import unittest_support as unittest
-from numba.cuda.testing import skip_on_cudasim
+from numba.cuda.testing import skip_on_cudasim, SerialMixin
 
 
 def set_a(ary, i, v):
@@ -100,7 +100,7 @@ recordwith2darray = np.dtype([('i', np.int32),
                               ('j', np.float32, (3, 2))])
 
 
-class TestRecordDtype(unittest.TestCase):
+class TestRecordDtype(SerialMixin, unittest.TestCase):
 
     def _createSampleArrays(self):
         self.sample1d = np.recarray(3, dtype=recordtype)
@@ -127,8 +127,15 @@ class TestRecordDtype(unittest.TestCase):
 
         for i in range(self.sample1d.size):
             got = self.sample1d.copy()
-            expect = np.recarray(got.shape, got.dtype)
-            expect[:] = got
+
+            # Force the argument to the pure Python function to be
+            # a recarray, as attribute access isn't supported on
+            # structured arrays.
+            if numpy_support.version <= (1, 9):
+                expect = np.recarray(got.shape, got.dtype)
+                expect[:] = got
+            else:
+                expect = got.copy().view(np.recarray)
 
             cfunc(got, i, value)
             pyfunc(expect, i, value)
@@ -265,6 +272,7 @@ class TestRecordDtype(unittest.TestCase):
 
         np.testing.assert_equal(rec['j'], arr)
 
+
 @skip_on_cudasim('Attribute access of structured arrays not supported in simulator')
 class TestRecordDtypeWithStructArrays(TestRecordDtype):
     '''
@@ -276,5 +284,6 @@ class TestRecordDtypeWithStructArrays(TestRecordDtype):
         self.samplerec1darr = np.zeros(1, dtype=recordwitharray)[0]
         self.samplerec2darr = np.zeros(1, dtype=recordwith2darray)[0]
 
+
 if __name__ == '__main__':
     unittest.main()
