@@ -94,6 +94,12 @@ class DataModel(object):
         """
         return []
 
+    def traverse_models(self):
+        """
+        Recursively list all models involved in this model.
+        """
+        return [self._dmm[t] for t in self.traverse_types()]
+
     def traverse_types(self):
         """
         Recursively list all frontend types involved in this model.
@@ -116,6 +122,12 @@ class DataModel(object):
     def has_nrt_meminfo(self):
         return False
 
+    def contains_nrt_meminfo(self):
+        """
+        Recursively check all contained types for need for NRT meminfo.
+        """
+        return any(model.has_nrt_meminfo() for model in self.traverse_models())
+
     def _compared_fields(self):
         return (type(self), self._fe_type)
 
@@ -269,6 +281,7 @@ class EnumModel(ProxyModel):
 @register_default(types.PyObject)
 @register_default(types.RawPointer)
 @register_default(types.NoneType)
+@register_default(types.Const)
 @register_default(types.EllipsisType)
 @register_default(types.Function)
 @register_default(types.Type)
@@ -287,7 +300,6 @@ class EnumModel(ProxyModel):
 @register_default(types.NumberClass)
 @register_default(types.NamedTupleClass)
 @register_default(types.DType)
-@register_default(types.ArrayFlags)
 @register_default(types.RecursiveCall)
 class OpaqueModel(PrimitiveModel):
     """
@@ -836,6 +848,16 @@ class ArrayModel(StructModel):
         ]
         super(ArrayModel, self).__init__(dmm, fe_type, members)
 
+
+@register_default(types.ArrayFlags)
+class ArrayFlagsModel(StructModel):
+    def __init__(self, dmm, fe_type):
+        members = [
+            ('parent', fe_type.array_type),
+        ]
+        super(ArrayFlagsModel, self).__init__(dmm, fe_type, members)
+
+
 @register_default(types.SmartArrayType)
 class SmartArrayModel(StructModel):
     def __init__(self, dmm, fe_type):
@@ -984,7 +1006,7 @@ class CContiguousFlatIter(StructModel):
         array_type = fe_type.array_type
         dtype = array_type.dtype
         ndim = array_type.ndim
-        members = [('array', types.EphemeralPointer(array_type)),
+        members = [('array', array_type),
                    ('stride', types.intp),
                    ('index', types.EphemeralPointer(types.intp)),
                    ]
@@ -999,7 +1021,7 @@ class FlatIter(StructModel):
         array_type = fe_type.array_type
         dtype = array_type.dtype
         ndim = array_type.ndim
-        members = [('array', types.EphemeralPointer(array_type)),
+        members = [('array', array_type),
                    ('pointers', types.EphemeralArray(types.CPointer(dtype), ndim)),
                    ('indices', types.EphemeralArray(types.intp, ndim)),
                    ('exhausted', types.EphemeralPointer(types.boolean)),
