@@ -4,11 +4,13 @@ Define @jit and related decorators.
 
 from __future__ import print_function, division, absolute_import
 
+import sys
 import warnings
 
 from . import config, sigutils
 from .errors import DeprecationError
 from .targets import registry
+from .stencil import stencil
 
 
 
@@ -25,14 +27,6 @@ def autojit(*args, **kws):
     return jit(*args, **kws)
 
 
-class _DisableJitWrapper(object):
-    def __init__(self, py_func):
-        self.py_func = py_func
-
-    def __call__(self, *args, **kwargs):
-        return self.py_func(*args, **kwargs)
-
-
 _msg_deprecated_signature_arg = ("Deprecated keyword argument `{0}`. "
                                  "Signatures should be passed as the first "
                                  "positional argument.")
@@ -40,7 +34,7 @@ _msg_deprecated_signature_arg = ("Deprecated keyword argument `{0}`. "
 def jit(signature_or_function=None, locals={}, target='cpu', cache=False, **options):
     """
     This decorator is used to compile a Python function into native code.
-    
+
     Args
     -----
     signature:
@@ -58,7 +52,7 @@ def jit(signature_or_function=None, locals={}, target='cpu', cache=False, **opti
         Specifies the target platform to compile for. Valid targets are cpu,
         gpu, npyufunc, and cuda. Defaults to cpu.
 
-    targetoptions:
+    options:
         For a cpu target, valid options are:
             nopython: bool
                 Set to True to disable the use of PyObjects and Python API
@@ -77,6 +71,12 @@ def jit(signature_or_function=None, locals={}, target='cpu', cache=False, **opti
                 mode. Any arrays that the tight loop uses should be created
                 before the loop is entered. Default value is True.
 
+            error_model: str
+                The error-model affects divide-by-zero behavior.
+                Valid values are 'python' and 'numpy'. The 'python' model
+                raises exception.  The 'numpy' model sets the result to
+                *+/-inf* or *nan*.
+
     Returns
     --------
     A callable usable as a compiled function.  Actual compiling will be
@@ -127,6 +127,22 @@ def jit(signature_or_function=None, locals={}, target='cpu', cache=False, **opti
     if 'restype' in options:
         raise DeprecationError(_msg_deprecated_signature_arg.format('restype'))
 
+    if options.get('parallel'):
+        uns1 = sys.platform.startswith('win32') and sys.version_info[:2] == (2, 7)
+        uns2 = sys.maxsize <= 2 ** 32
+        if uns1 or uns2:
+            msg = ("The 'parallel' target is not currently supported on "
+                   "Windows operating systems when using Python 2.7, or "
+                   "on 32 bit hardware.")
+            raise RuntimeError(msg)
+        if cache:
+            msg = ("Caching is not available when the 'parallel' target is in "
+                   "use. Caching is now being disabled to allow execution to "
+                   "continue.")
+            warnings.warn(msg, RuntimeWarning)
+            cache = False
+
+
     # Handle signature
     if signature_or_function is None:
         # No signature, no function
@@ -161,7 +177,7 @@ def _jit(sigs, locals, target, cache, targetoptions, **dispatcher_args):
             from . import cuda
             return cuda.jit(func)
         if config.DISABLE_JIT and not target == 'npyufunc':
-            return _DisableJitWrapper(func)
+            return func
         disp = dispatcher(py_func=func, locals=locals,
                           targetoptions=targetoptions,
                           **dispatcher_args)
