@@ -9,7 +9,6 @@ import struct
 import sys
 import uuid
 import weakref
-import threading
 
 import numba
 from numba import _dispatcher, compiler, utils, types, config, errors
@@ -135,6 +134,28 @@ _CompileStats = collections.namedtuple(
     '_CompileStats', ('cache_path', 'cache_hits', 'cache_misses'))
 
 
+class _CompilingCounter(object):
+    """
+    A simple counter that increment in __enter__ and decrement in __exit__.
+    """
+
+    def __init__(self):
+        self.counter = 0
+
+    def __enter__(self):
+        assert self.counter >= 0
+        self.counter += 1
+
+    def __exit__(self, *args, **kwargs):
+        self.counter -= 1
+        assert self.counter >= 0
+
+    def __bool__(self):
+        return self.counter > 0
+
+    __nonzero__ = __bool__
+
+
 class _DispatcherBase(_dispatcher.Dispatcher):
     """
     Common base class for dispatcher Implementations.
@@ -142,7 +163,7 @@ class _DispatcherBase(_dispatcher.Dispatcher):
 
     __numba__ = "py_func"
 
-    def __init__(self, arg_count, py_func, pysig):
+    def __init__(self, arg_count, py_func, pysig, can_fallback):
         self._tm = default_type_manager
 
         # A mapping of signatures to compile results
@@ -166,11 +187,11 @@ class _DispatcherBase(_dispatcher.Dispatcher):
         _dispatcher.Dispatcher.__init__(self, self._tm.get_pointer(),
                                         arg_count, self._fold_args,
                                         argnames, defargs,
+                                        can_fallback,
                                         has_stararg)
 
         self.doc = py_func.__doc__
-        self._compile_lock = threading.RLock()
-
+        self._compiling_counter = _CompilingCounter()
         utils.finalize(self, self._make_finalizer())
 
     def _reset_overloads(self):
@@ -268,7 +289,7 @@ class _DispatcherBase(_dispatcher.Dispatcher):
         """
         Whether a specialization is currently being compiled.
         """
-        return self._compile_lock._is_owned()
+        return self._compiling_counter
 
     def _compile_for_args(self, *args, **kws):
         """
@@ -332,6 +353,28 @@ class _DispatcherBase(_dispatcher.Dispatcher):
             print(res.type_annotation, file=file)
             print('=' * 80, file=file)
 
+    def inspect_cfg(self, signature=None, show_wrapper=None):
+        """
+        For inspecting the CFG of the function.
+
+        By default the CFG of the user function is showed.  The *show_wrapper*
+        option can be set to "python" or "cfunc" to show the python wrapper
+        function or the *cfunc* wrapper function, respectively.
+        """
+        if signature is not None:
+            cres = self.overloads[signature]
+            lib = cres.library
+            if show_wrapper == 'python':
+                fname = cres.fndesc.llvm_cpython_wrapper_name
+            elif show_wrapper == 'cfunc':
+                fname = cres.fndesc.llvm_cfunc_wrapper_name
+            else:
+                fname = cres.fndesc.mangled_name
+            return lib.get_function_cfg(fname)
+
+        return dict((sig, self.inspect_cfg(sig, show_wrapper=show_wrapper))
+                    for sig in self.signatures)
+
     def _explain_ambiguous(self, *args, **kws):
         """
         Callback for the C _Dispatcher object.
@@ -407,6 +450,7 @@ class Dispatcher(_DispatcherBase):
     # A {uuid -> instance} mapping, for deserialization
     _memo = weakref.WeakValueDictionary()
     __uuid = None
+    __numba__ = 'py_func'
 
     def __init__(self, py_func, locals={}, targetoptions={}, impl_kind='direct'):
         """
@@ -424,8 +468,8 @@ class Dispatcher(_DispatcherBase):
 
         pysig = utils.pysignature(py_func)
         arg_count = len(pysig.parameters)
-
-        _DispatcherBase.__init__(self, arg_count, py_func, pysig)
+        can_fallback = not targetoptions.get('nopython', False)
+        _DispatcherBase.__init__(self, arg_count, py_func, pysig, can_fallback)
 
         functools.update_wrapper(self, py_func)
 
@@ -510,30 +554,33 @@ class Dispatcher(_DispatcherBase):
     def compile(self, sig):
         if not self._can_compile:
             raise RuntimeError("compilation disabled")
-        with self._compile_lock:
-            args, return_type = sigutils.normalize_signature(sig)
-            # Don't recompile if signature already exists
-            existing = self.overloads.get(tuple(args))
-            if existing is not None:
-                return existing.entry_point
+        # Use cache and compiler in a critical section
+        with compiler.lock_compiler:
+            # Use counter to track recursion compilation depth
+            with self._compiling_counter:
+                args, return_type = sigutils.normalize_signature(sig)
+                # Don't recompile if signature already exists
+                existing = self.overloads.get(tuple(args))
+                if existing is not None:
+                    return existing.entry_point
 
-            # Try to load from disk cache
-            cres = self._cache.load_overload(sig, self.targetctx)
-            if cres is not None:
-                self._cache_hits[sig] += 1
-                # XXX fold this in add_overload()? (also see compiler.py)
-                if not cres.objectmode and not cres.interpmode:
-                    self.targetctx.insert_user_function(cres.entry_point,
-                                                   cres.fndesc, [cres.library])
+                # Try to load from disk cache
+                cres = self._cache.load_overload(sig, self.targetctx)
+                if cres is not None:
+                    self._cache_hits[sig] += 1
+                    # XXX fold this in add_overload()? (also see compiler.py)
+                    if not cres.objectmode and not cres.interpmode:
+                        self.targetctx.insert_user_function(cres.entry_point,
+                                                    cres.fndesc, [cres.library])
+                    self.add_overload(cres)
+                    return cres.entry_point
+
+                self._cache_misses[sig] += 1
+                cres = self._compiler.compile(args, return_type)
                 self.add_overload(cres)
+                self._cache.save_overload(sig, cres)
                 return cres.entry_point
 
-            self._cache_misses[sig] += 1
-            cres = self._compiler.compile(args, return_type)
-            self.add_overload(cres)
-            self._cache.save_overload(sig, cres)
-            return cres.entry_point
-
     def recompile(self):
         """
         Recompile all signatures afresh.
@@ -578,7 +625,8 @@ class LiftedLoop(_DispatcherBase):
 
         _DispatcherBase.__init__(self, self.func_ir.arg_count,
                                  self.func_ir.func_id.func,
-                                 self.func_ir.func_id.pysig)
+                                 self.func_ir.func_id.pysig,
+                                 can_fallback=True)
 
     def get_source_location(self):
         """Return the starting line number of the loop.
@@ -586,32 +634,37 @@ class LiftedLoop(_DispatcherBase):
         return self.func_ir.loc.line
 
     def compile(self, sig):
-        with self._compile_lock:
-            # XXX this is mostly duplicated from Dispatcher.
-            flags = self.flags
-            args, return_type = sigutils.normalize_signature(sig)
+        # Use cache and compiler in a critical section
+        with compiler.lock_compiler:
+            # Use counter to track recursion compilation depth
+            with self._compiling_counter:
+                # XXX this is mostly duplicated from Dispatcher.
+                flags = self.flags
+                args, return_type = sigutils.normalize_signature(sig)
 
-            # Don't recompile if signature already exists
-            # (e.g. if another thread compiled it before we got the lock)
-            existing = self.overloads.get(tuple(args))
-            if existing is not None:
-                return existing.entry_point
+                # Don't recompile if signature already exists
+                # (e.g. if another thread compiled it before we got the lock)
+                existing = self.overloads.get(tuple(args))
+                if existing is not None:
+                    return existing.entry_point
 
-            assert not flags.enable_looplift, "Enable looplift flags is on"
-            cres = compiler.compile_ir(typingctx=self.typingctx,
-                                       targetctx=self.targetctx,
-                                       func_ir=self.func_ir,
-                                       args=args, return_type=return_type,
-                                       flags=flags, locals=self.locals,
-                                       lifted=(),
-                                       lifted_from=self.lifted_from)
+                assert not flags.enable_looplift, "Enable looplift flags is on"
+                # Clone IR to avoid mutation in rewrite pass
+                cloned_func_ir = self.func_ir.copy()
+                cres = compiler.compile_ir(typingctx=self.typingctx,
+                                           targetctx=self.targetctx,
+                                           func_ir=cloned_func_ir,
+                                           args=args, return_type=return_type,
+                                           flags=flags, locals=self.locals,
+                                           lifted=(),
+                                           lifted_from=self.lifted_from)
 
-            # Check typing error if object mode is used
-            if cres.typing_error is not None and not flags.enable_pyobject:
-                raise cres.typing_error
+                # Check typing error if object mode is used
+                if cres.typing_error is not None and not flags.enable_pyobject:
+                    raise cres.typing_error
 
-            self.add_overload(cres)
-            return cres.entry_point
+                self.add_overload(cres)
+                return cres.entry_point
 
 
 # Initialize typeof machinery
