@@ -9,6 +9,7 @@ import sys
 from . import utils
 from .errors import (NotDefinedError, RedefinedError, VerificationError,
                      ConstantInferenceError)
+from numba import config
 
 
 class Loc(object):
@@ -264,17 +265,27 @@ class Expr(Inst):
         op = 'cast'
         return cls(op=op, value=value, loc=loc)
 
+    @classmethod
+    def make_function(cls, name, code, closure, defaults, loc):
+        """
+        A node for making a function object.
+        """
+        op = 'make_function'
+        return cls(op=op, name=name, code=code, closure=closure, defaults=defaults, loc=loc)
+
     def __repr__(self):
         if self.op == 'call':
             args = ', '.join(str(a) for a in self.args)
-            kws = ', '.join('%s=%s' % (k, v) for k, v in self.kws)
+            pres_order = self._kws.items() if config.DIFF_IR == 0 else sorted(self._kws.items())
+            kws = ', '.join('%s=%s' % (k, v) for k, v in pres_order)
             vararg = '*%s' % (self.vararg,) if self.vararg is not None else ''
             arglist = ', '.join(filter(None, [args, vararg, kws]))
             return 'call %s(%s)' % (self.func, arglist)
         elif self.op == 'binop':
             return '%s %s %s' % (self.lhs, self.fn, self.rhs)
         else:
-            args = ('%s=%s' % (k, v) for k, v in self._kws.items())
+            pres_order = self._kws.items() if config.DIFF_IR == 0 else sorted(self._kws.items())
+            args = ('%s=%s' % (k, v) for k, v in pres_order)
             return '%s(%s)' % (self.op, ', '.join(args))
 
     def list_vars(self):
@@ -526,7 +537,6 @@ class Const(object):
     def infer_constant(self):
         return self.value
 
-
 class Global(object):
     def __init__(self, name, value, loc):
         self.name = name
@@ -539,6 +549,11 @@ class Global(object):
     def infer_constant(self):
         return self.value
 
+    def __deepcopy__(self, memo):
+        # don't copy value since it can fail (e.g. modules)
+        # value is readonly and doesn't need copying
+        return Global(self.name, self.value, copy.deepcopy(self.loc))
+
 
 class FreeVar(object):
     """
@@ -644,10 +659,17 @@ class Scope(object):
 
     def get(self, name):
         """
-        Refer to a variable
+        Refer to a variable.  Returns the latest version.
         """
         if name in self.redefined:
             name = "%s.%d" % (name, self.redefined[name])
+        return self.get_exact(name)
+
+    def get_exact(self, name):
+        """
+        Refer to a variable.  The returned variable has the exact
+        name (exact variable version).
+        """
         try:
             return self.localvars.get(name)
         except NotDefinedError:
@@ -666,12 +688,16 @@ class Scope(object):
         else:
             return self.localvars.get(name)
 
-    def redefine(self, name, loc):
+    def redefine(self, name, loc, rename=True):
         """
         Redefine if the name is already defined
         """
         if name not in self.localvars:
             return self.define(name, loc)
+        elif not rename:
+            # Must use the same name if the variable is a cellvar, which
+            # means it could be captured in a closure.
+            return self.localvars.get(name)
         else:
             ct = self.redefined[name]
             self.redefined[name] = ct + 1
@@ -747,8 +773,11 @@ class Block(object):
         # Avoid early bind of sys.stdout as default value
         file = file or sys.stdout
         for inst in self.body:
-            inst_vars = sorted(str(v) for v in inst.list_vars())
-            print('    %-40s %s' % (inst, inst_vars), file=file)
+            if hasattr(inst, 'dump'):
+                inst.dump(file)
+            else:
+                inst_vars = sorted(str(v) for v in inst.list_vars())
+                print('    %-40s %s' % (inst, inst_vars), file=file)
 
     @property
     def terminator(self):
@@ -845,6 +874,19 @@ class FunctionIR(object):
 
         return new_ir
 
+    def copy(self):
+        new_ir = copy.copy(self)
+        blocks = {}
+        block_entry_vars = {}
+        for label, block in self.blocks.items():
+            new_block = block.copy()
+            blocks[label] = new_block
+            if block in self.block_entry_vars:
+                block_entry_vars[new_block] = self.block_entry_vars[block]
+        new_ir.blocks = blocks
+        new_ir.block_entry_vars = block_entry_vars
+        return new_ir
+
     def get_block_entry_vars(self, block):
         """
         Return a set of variable names possibly alive at the beginning of
@@ -860,18 +902,22 @@ class FunctionIR(object):
             name = name.name
         return self._consts.infer_constant(name)
 
-    def get_definition(self, value):
+    def get_definition(self, value, lhs_only=False):
         """
         Get the definition site for the given variable name or instance.
-        A Expr instance is returned.
+        A Expr instance is returned by default, but if lhs_only is set
+        to True, the left-hand-side variable is returned instead.
         """
+        lhs = value
         while True:
             if isinstance(value, Var):
+                lhs = value
                 name = value.name
             elif isinstance(value, str):
+                lhs = value
                 name = value
             else:
-                return value
+                return lhs if lhs_only else value
             defs = self._definitions[name]
             if len(defs) == 0:
                 raise KeyError("no definition for %r"
