@@ -1,17 +1,16 @@
 from __future__ import print_function, division, absolute_import
 
 from collections import namedtuple
-import sys
 from functools import partial
 
-from llvmlite.ir import Value
 from llvmlite.llvmpy.core import Constant, Type, Builder
 
 from . import (_dynfunc, cgutils, config, funcdesc, generators, ir, types,
                typing, utils)
 from .errors import LoweringError, new_error_context
-from .targets import imputils
+from .targets import removerefctpass
 from .funcdesc import default_mangler
+from . import debuginfo
 
 
 class Environment(_dynfunc.Environment):
@@ -25,6 +24,15 @@ class Environment(_dynfunc.Environment):
     def __reduce__(self):
         return _rebuild_env, (self.globals['__name__'], self.consts)
 
+    def as_pointer(self, targetctx, ptrty=types.pyobject):
+        """
+        Return a constant pointer for the environment object.
+        """
+        ll_addr = targetctx.get_value_type(types.intp)
+        ll_ptr = targetctx.get_value_type(ptrty)
+        envptr = ll_addr(id(self)).inttoptr(ll_ptr)
+        return envptr
+
 
 def _rebuild_env(modname, consts):
     from . import serialize
@@ -41,12 +49,7 @@ class BaseLower(object):
     """
     Lower IR to LLVM
     """
-
-    # If true, then can't cache LLVM module accross process calls
-    has_dynamic_globals = False
-
     def __init__(self, context, library, fndesc, func_ir):
-        self.context = context
         self.library = library
         self.fndesc = fndesc
         self.blocks = utils.SortedMap(utils.iteritems(func_ir.blocks))
@@ -67,6 +70,19 @@ class BaseLower(object):
         self.firstblk = min(self.blocks.keys())
         self.loc = -1
 
+        # Specializes the target context as seen inside the Lowerer
+        # This adds:
+        #  - environment: the python exceution environment
+        self.context = context.subtarget(environment=self.env)
+
+        # Debuginfo
+        dibuildercls = (self.context.DIBuilder
+                        if self.context.enable_debuginfo
+                        else debuginfo.DummyDIBuilder)
+
+        self.debuginfo = dibuildercls(module=self.module,
+                                      filepath=func_ir.loc.filename)
+
         # Subclass initialization
         self.init()
 
@@ -83,20 +99,9 @@ class BaseLower(object):
         self.pyapi = self.context.get_python_api(self.builder)
 
         # Store environment argument for later use
-        self.envarg = self.call_conv.get_env_argument(self.function)
-        # Sanity check
-        with cgutils.if_unlikely(self.builder,
-                                 cgutils.is_null(self.builder, self.envarg)):
-            self.pyapi.err_set_string(
-                "PyExc_SystemError",
-                "Numba internal error: object mode function called "
-                "without an environment")
-            self.call_conv.return_exc(self.builder)
-
-        self.env_body = self.context.get_env_body(self.builder, self.envarg)
-        self.pyapi.emit_environment_sentry(self.envarg)
-        self.env_manager = self.pyapi.get_env_manager(self.env, self.env_body,
-                                                      self.envarg)
+        self.env_manager = self.context.get_env_manager(self.builder)
+        self.env_body = self.env_manager.env_body
+        self.envarg = self.env_manager.env_ptr
 
     def pre_lower(self):
         """
@@ -106,11 +111,15 @@ class BaseLower(object):
         # (for generators) and it's important to use a new API and
         # EnvironmentManager.
         self.pyapi = None
+        self.debuginfo.mark_subprogram(function=self.builder.function,
+                                       name=self.fndesc.qualname,
+                                       loc=self.func_ir.loc)
 
     def post_lower(self):
         """
         Called after all blocks are lowered
         """
+        self.debuginfo.finalize()
 
     def pre_block(self, block):
         """
@@ -138,6 +147,12 @@ class BaseLower(object):
             print(self.module)
             print('=' * 80)
 
+        # Special optimization to remove NRT on functions that do not need it.
+        if self.context.enable_nrt and self.generator_info is None:
+            removerefctpass.remove_unnecessary_nrt_usage(self.function,
+                                                         context=self.context,
+                                                         fndesc=self.fndesc)
+
         # Run target specific post lowering transformation
         self.context.post_lowering(self.module, self.library)
 
@@ -227,11 +242,23 @@ class BaseLower(object):
         if config.DEBUG_JIT:
             self.context.debug_print(self.builder, "DEBUGJIT: {0}".format(msg))
 
+    @property
+    def has_dynamic_globals(self):
+        """
+        If true, then can't cache LLVM module accross process calls.
+        """
+        return self.library.has_dynamic_globals
+
+
+# Dictionary mapping instruction class to its lowering function.
+lower_extensions = {}
 
 class Lower(BaseLower):
     GeneratorLower = generators.GeneratorLower
 
     def lower_inst(self, inst):
+        # Set debug location for all subsequent LL instructions
+        self.debuginfo.mark_location(self.builder, self.loc)
         self.debug_print(str(inst))
         if isinstance(inst, ir.Assign):
             ty = self.typeof(inst.target.name)
@@ -333,6 +360,10 @@ class Lower(BaseLower):
             self.lower_static_raise(inst)
 
         else:
+            for _class, func in lower_extensions.items():
+                if isinstance(inst, _class):
+                    func(self, inst)
+                    return
             raise NotImplementedError(type(inst))
 
     def lower_setitem(self, target_var, index_var, value_var, signature):
@@ -371,9 +402,6 @@ class Lower(BaseLower):
         value = inst.value
         # In nopython mode, closure vars are frozen like globals
         if isinstance(value, (ir.Const, ir.Global, ir.FreeVar)):
-            if isinstance(ty, types.ExternalFunctionPointer):
-                self.has_dynamic_globals = True
-
             res = self.context.get_constant_generic(self.builder, ty,
                                                     value.value)
             self.incref(ty, res)
@@ -422,8 +450,16 @@ class Lower(BaseLower):
         # Yield to caller
         val = self.loadvar(inst.value.name)
         typ = self.typeof(inst.value.name)
-        val = self.context.cast(self.builder, val, typ, self.gentype.yield_type)
-        self.call_conv.return_value(self.builder, val)
+
+        # cast the local val to the type yielded
+        yret = self.context.cast(self.builder, val, typ,
+                                    self.gentype.yield_type)
+
+        # get the return repr of yielded value
+        retval = self.context.get_return_value(self.builder, typ, yret)
+        
+        # return
+        self.call_conv.return_value(self.builder, retval)
 
         # Resumption point
         y.lower_yield_resume()
@@ -579,6 +615,7 @@ class Lower(BaseLower):
 
     def lower_call(self, resty, expr):
         signature = self.fndesc.calltypes[expr]
+        self.debug_print("# lower_call: expr = {0}".format(expr))
         if isinstance(signature.return_type, types.Phantom):
             return self.context.get_dummy_value()
 
@@ -660,6 +697,7 @@ class Lower(BaseLower):
         else:
             # Normal function resolution
             self.debug_print("# calling normal function: {0}".format(fnty))
+            self.debug_print("# signature: {0}".format(signature))
             impl = self.context.get_function(fnty, signature)
             if signature.recvr:
                 # The "self" object is passed as the function object
@@ -734,6 +772,11 @@ class Lower(BaseLower):
         elif expr.op == 'exhaust_iter':
             val = self.loadvar(expr.value.name)
             ty = self.typeof(expr.value.name)
+            # Unpack optional
+            if isinstance(ty, types.Optional):
+                val = self.context.cast(self.builder, val, ty, ty.type)
+                ty = ty.type
+
             # If we have a tuple, we needn't do anything
             # (and we can't iterate over the heterogenous ones).
             if isinstance(ty, types.BaseTuple):
@@ -928,7 +971,17 @@ class Lower(BaseLower):
         return self.alloca_lltype(name, lltype)
 
     def alloca_lltype(self, name, lltype):
-        return cgutils.alloca_once(self.builder, lltype, name=name, zfill=True)
+        # Is user variable?
+        is_uservar = not name.startswith('$')
+        # Allocate space for variable
+        aptr = cgutils.alloca_once(self.builder, lltype, name=name, zfill=True)
+        if is_uservar:
+            # Emit debug info for user variable
+            sizeof = self.context.get_abi_sizeof(lltype)
+            self.debuginfo.mark_variable(self.builder, aptr, name=name,
+                                         lltype=lltype, size=sizeof,
+                                         loc=self.loc)
+        return aptr
 
     def incref(self, typ, val):
         if not self.context.enable_nrt:
