@@ -3,6 +3,7 @@ from __future__ import print_function, division, absolute_import
 
 import warnings
 import inspect
+from contextlib import contextmanager
 
 import numpy as np
 
@@ -15,6 +16,7 @@ from numba.numpy_support import as_dtype
 from . import _internal
 from .sigparse import parse_signature
 from .wrappers import build_ufunc_wrapper, build_gufunc_wrapper
+from numba.caching import FunctionCache, NullCache
 
 
 import llvmlite.llvmpy.core as lc
@@ -23,6 +25,7 @@ class UFuncTargetOptions(TargetOptions):
     OPTIONS = {
         "nopython" : bool,
         "forceobj" : bool,
+        "fastmath" : bool,
     }
 
 
@@ -51,6 +54,10 @@ class UFuncDispatcher(object):
         self.overloads = utils.UniqueDict()
         self.targetoptions = targetoptions
         self.locals = locals
+        self.cache = NullCache()
+
+    def enable_caching(self):
+        self.cache = FunctionCache(self.py_func)
 
     def compile(self, sig, locals={}, **targetoptions):
         locs = self.locals.copy()
@@ -61,24 +68,54 @@ class UFuncDispatcher(object):
 
         flags = compiler.Flags()
         self.targetdescr.options.parse_as_flags(flags, topt)
-        flags.set("no_compile")
+
         flags.set("no_cpython_wrapper")
         flags.set("error_model", "numpy")
         # Disable loop lifting
         # The feature requires a real python function
         flags.unset("enable_looplift")
 
+        return self._compile_core(sig, flags, locals)
+
+    def _compile_core(self, sig, flags, locals):
+        """
+        Trigger the compiler on the core function or load a previously
+        compiled version from the cache.  Returns the CompileResult.
+        """
         typingctx = self.targetdescr.typing_context
         targetctx = self.targetdescr.target_context
 
-        args, return_type = sigutils.normalize_signature(sig)
+        @contextmanager
+        def store_overloads_on_success():
+            # use to ensure overloads are stored on success
+            try:
+                yield
+            except:
+                raise
+            else:
+                exists = self.overloads.get(cres.signature)
+                if exists is None:
+                    self.overloads[cres.signature] = cres
 
-        cres = compiler.compile_extra(typingctx, targetctx, self.py_func,
-                                      args=args, return_type=return_type,
-                                      flags=flags, locals=locals)
+        # Use cache and compiler in a critical section
+        with compiler.lock_compiler:
+            with store_overloads_on_success():
+                # attempt look up of existing
+                cres = self.cache.load_overload(sig, targetctx)
+                if cres is not None:
+                    return cres
 
-        self.overloads[cres.signature] = cres
-        return cres
+                # Compile
+                args, return_type = sigutils.normalize_signature(sig)
+                cres = compiler.compile_extra(typingctx, targetctx,
+                                              self.py_func, args=args,
+                                              return_type=return_type,
+                                              flags=flags, locals=locals)
+
+                # cache lookup failed before so safe to save
+                self.cache.save_overload(sig, cres)
+
+                return cres
 
 
 dispatcher_registry['npyufunc'] = UFuncDispatcher
@@ -114,22 +151,14 @@ def _build_element_wise_ufunc_wrapper(cres, signature):
     '''
     ctx = cres.target_context
     library = cres.library
-    llvm_func = library.get_function(cres.fndesc.llvm_func_name)
+    fname = cres.fndesc.llvm_func_name
 
-    env = None
-    if cres.objectmode:
-        # Get env
-        env = cres.environment
-        assert env is not None
-        ll_intp = cres.target_context.get_value_type(types.intp)
-        ll_pyobj = cres.target_context.get_value_type(types.pyobject)
-        envptr = lc.Constant.int(ll_intp, id(env)).inttoptr(ll_pyobj)
-    else:
-        envptr = None
+    env = cres.environment
+    envptr = env.as_pointer(ctx)
 
-    wrapper = build_ufunc_wrapper(library, ctx, llvm_func, signature,
-                                  cres.objectmode, envptr, env)
-    ptr = library.get_pointer_to_function(wrapper.name)
+    with compiler.lock_compiler:
+        ptr = build_ufunc_wrapper(library, ctx, fname, signature,
+                                cres.objectmode, envptr, env)
 
     # Get dtypes
     dtypenums = [as_dtype(a).num for a in signature.args]
@@ -181,10 +210,10 @@ class _BaseUFuncBuilder(object):
 
 class UFuncBuilder(_BaseUFuncBuilder):
 
-    def __init__(self, py_func, identity=None, targetoptions={}):
+    def __init__(self, py_func, identity=None, cache=False, targetoptions={}):
         self.py_func = py_func
         self.identity = parse_identity(identity)
-        self.nb_func = jit(target='npyufunc', **targetoptions)(py_func)
+        self.nb_func = jit(target='npyufunc', cache=cache, **targetoptions)(py_func)
         self._sigs = []
         self._cres = {}
 
@@ -240,13 +269,15 @@ class UFuncBuilder(_BaseUFuncBuilder):
 class GUFuncBuilder(_BaseUFuncBuilder):
 
     # TODO handle scalar
-    def __init__(self, py_func, signature, identity=None, targetoptions={}):
+    def __init__(self, py_func, signature, identity=None, cache=False,
+                 targetoptions={}):
         self.py_func = py_func
         self.identity = parse_identity(identity)
-        self.nb_func = jit(target='npyufunc')(py_func)
+        self.nb_func = jit(target='npyufunc', cache=cache)(py_func)
         self.signature = signature
         self.sin, self.sout = parse_signature(signature)
         self.targetoptions = targetoptions
+        self.cache = cache
         self._sigs = []
         self._cres = {}
 
@@ -290,16 +321,11 @@ class GUFuncBuilder(_BaseUFuncBuilder):
         Returns (dtype numbers, function ptr, EnvironmentObject)
         """
         # Buider wrapper for ufunc entry point
-        ctx = cres.target_context
-        library = cres.library
         signature = cres.signature
-        llvm_func = library.get_function(cres.fndesc.llvm_func_name)
-        wrapper, env = build_gufunc_wrapper(library, ctx, llvm_func,
-                                            signature, self.sin, self.sout,
-                                            fndesc=cres.fndesc,
-                                            env=cres.environment)
-
-        ptr = library.get_pointer_to_function(wrapper.name)
+        with compiler.lock_compiler:
+            ptr, env, wrapper_name = build_gufunc_wrapper(self.py_func, cres,
+                                                          self.sin, self.sout,
+                                                          cache=self.cache)
 
         # Get dtypes
         dtypenums = []
