@@ -25,6 +25,7 @@ race condition.
 #include <stdio.h>
 #include "workqueue.h"
 #include "../_pymodule.h"
+#include "gufunc_scheduler.h"
 
 /* As the thread-pool isn't inherited by children,
    free the task-queue, too. */
@@ -211,8 +212,8 @@ queue_state_wait(Queue *queue, int old, int repl)
         queue_condition_wait(cond);
     }
     queue->state = repl;
-    queue_condition_unlock(cond);
     queue_condition_signal(cond);
+    queue_condition_unlock(cond);
 }
 
 static void
@@ -306,6 +307,8 @@ MOD_INIT(workqueue) {
                            PyLong_FromVoidPtr(&ready));
     PyObject_SetAttrString(m, "add_task",
                            PyLong_FromVoidPtr(&add_task));
+    PyObject_SetAttrString(m, "do_scheduling",
+                           PyLong_FromVoidPtr(&do_scheduling));
 
     return MOD_SUCCESS_VAL(m);
 }
