@@ -13,7 +13,7 @@ import llvmlite.llvmpy.core as lc
 from llvmlite.llvmpy.core import Type, Constant, LLVMException
 import llvmlite.binding as ll
 
-from numba import types, utils, cgutils, typing
+from numba import types, utils, cgutils, typing, funcdesc, debuginfo
 from numba import _dynfunc, _helperlib
 from numba.pythonapi import PythonAPI
 from . import arrayobj, builtins, imputils
@@ -22,7 +22,6 @@ from .imputils import (user_function, user_generator,
                        RegistryLoader)
 from numba import datamodel
 
-
 GENERIC_POINTER = Type.pointer(Type.int(8))
 PYOBJECT = GENERIC_POINTER
 void_ptr = GENERIC_POINTER
@@ -187,25 +186,38 @@ class BaseContext(object):
     # Causes exception to be raised if the record members are not aligned.
     strict_alignment = False
 
-    # Use default mangler (no specific requirement)
-    mangler = None
-
     # Force powi implementation as math.pow call
     implement_powi_as_math_call = False
     implement_pow_as_math_call = False
 
+    # Emit Debug info
+    enable_debuginfo = False
+    DIBuilder = debuginfo.DIBuilder
+
     # Bound checking
     enable_boundcheck = False
 
     # NRT
     enable_nrt = False
 
+    # Auto parallelization
+    auto_parallel = False
+
     # PYCC
     aot_mode = False
 
     # Error model for various operations (only FP exceptions currently)
     error_model = None
 
+    # Whether dynamic globals (CPU runtime addresses) is allowed
+    allow_dynamic_globals = False
+
+    # Fast math flags
+    enable_fastmath = False
+
+    # python exceution environment
+    environment = None
+
     def __init__(self, typing_context):
         _load_global_helpers()
 
@@ -260,6 +272,12 @@ class BaseContext(object):
         Load target-specific registries.  Can be overriden by subclasses.
         """
 
+    def mangler(self, name, types):
+        """
+        Perform name mangling.
+        """
+        return funcdesc.default_mangler(name, types)
+
     def get_arg_packer(self, fe_args):
         return datamodel.ArgPacker(self.data_model_manager, fe_args)
 
@@ -384,11 +402,12 @@ class BaseContext(object):
         Insert a unique internal constant named *name*, with LLVM value
         *val*, into module *mod*.
         """
-        gv = mod.get_global(name)
-        if gv is not None:
-            return gv
-        else:
+        try:
+            gv = mod.get_global(name)
+        except KeyError:
             return cgutils.global_constant(mod, name, val)
+        else:
+            return gv
 
     def get_argument_type(self, ty):
         return self.data_model_manager[ty].get_argument_type()
@@ -910,18 +929,28 @@ class BaseContext(object):
         Create an array structure reifying the given constant array.
         A low-level contiguous array constant is created in the LLVM IR.
         """
-        assert typ.layout == 'C'                # assumed in typeinfer.py
         datatype = self.get_data_type(typ.dtype)
+        # don't freeze ary of non-contig or bigger than 1MB
+        size_limit = 10**6
 
-        # Handle data: reify the flattened array in "C" order as a
-        # global array of bytes.
-        flat = ary.flatten()
-        # Note: we use `bytearray(flat.data)` instead of `bytearray(flat)` to
-        #       workaround issue #1850 which is due to numpy issue #3147
-        consts = Constant.array(Type.int(8), bytearray(flat.data))
-        data = cgutils.global_constant(builder, ".const.array.data", consts)
-        # Ensure correct data alignment (issue #1933)
-        data.align = self.get_abi_alignment(datatype)
+        if (self.allow_dynamic_globals and
+                (typ.layout not in 'FC' or ary.nbytes > size_limit)):
+            # get pointer from the ary
+            dataptr = ary.ctypes.data
+            data = self.add_dynamic_addr(builder, dataptr, info=str(type(dataptr)))
+            rt_addr = self.add_dynamic_addr(builder, id(ary), info=str(type(ary)))
+        else:
+            # Handle data: reify the flattened array in "C" or "F" order as a
+            # global array of bytes.
+            flat = ary.flatten(order=typ.layout)
+            # Note: we use `bytearray(flat.data)` instead of `bytearray(flat)` to
+            #       workaround issue #1850 which is due to numpy issue #3147
+            consts = Constant.array(Type.int(8), bytearray(flat.data))
+            data = cgutils.global_constant(builder, ".const.array.data", consts)
+            # Ensure correct data alignment (issue #1933)
+            data.align = self.get_abi_alignment(datatype)
+            # No reference to parent ndarray
+            rt_addr = None
 
         # Handle shape
         llintp = self.get_value_type(types.intp)
@@ -929,20 +958,12 @@ class BaseContext(object):
         cshape = Constant.array(llintp, shapevals)
 
         # Handle strides
-        if ary.ndim > 0:
-            # Use strides of the equivalent C-contiguous array.
-            contig = np.ascontiguousarray(ary)
-            stridevals = [self.get_constant(types.intp, s) for s in contig.strides]
-        else:
-            stridevals = []
+        stridevals = [self.get_constant(types.intp, s) for s in ary.strides]
         cstrides = Constant.array(llintp, stridevals)
 
         # Create array structure
         cary = self.make_array(typ)(self, builder)
 
-        rt_addr = self.get_constant(types.uintp, id(ary)).inttoptr(
-            self.get_value_type(types.pyobject))
-
         intp_itemsize = self.get_constant(types.intp, ary.dtype.itemsize)
         self.populate_array(cary,
                             data=builder.bitcast(data, cary.data.type),
@@ -954,6 +975,27 @@ class BaseContext(object):
 
         return cary._getvalue()
 
+    def add_dynamic_addr(self, builder, intaddr, info):
+        """
+        Returns dynamic address as a void pointer `i8*`.
+
+        Internally, a global variable is added to inform the lowerer about
+        the usage of dynamic addresses.  Caching will be disabled.
+        """
+        assert self.allow_dynamic_globals, "dyn globals disabled in this target"
+        assert isinstance(intaddr, utils.INT_TYPES), 'dyn addr not of int type'
+        mod = builder.module
+        llvoidptr = self.get_value_type(types.voidptr)
+        addr = self.get_constant(types.uintp, intaddr).inttoptr(llvoidptr)
+        # Use a unique name by embedding the address value
+        symname = 'numba.dynamic.globals.{:x}'.format(intaddr)
+        gv = mod.add_global_variable(llvoidptr, name=symname)
+        # Use linkonce linkage to allow merging with other GV of the same name.
+        # And, avoid optimization from assuming its value.
+        gv.linkage = 'linkonce'
+        gv.initializer = addr
+        return builder.load(gv)
+
     def get_abi_sizeof(self, ty):
         """
         Get the ABI size of LLVM type *ty*.
@@ -985,7 +1027,6 @@ class BaseContext(object):
         return lc.Module(name)
 
 
-
 class _wrap_impl(object):
     """
     A wrapper object to call an implementation function with some predefined
