@@ -1,5 +1,6 @@
 from __future__ import print_function, division, absolute_import
 
+import warnings
 import functools
 import locale
 import weakref
@@ -13,10 +14,12 @@ import llvmlite.ir as llvmir
 
 from numba import config, utils, cgutils
 from numba.runtime.nrtopt import remove_redundant_nrt_refct
+from numba.runtime import rtsys
 
 _x86arch = frozenset(['x86', 'i386', 'i486', 'i586', 'i686', 'i786',
                       'i886', 'i986'])
 
+
 def _is_x86(triple):
     arch = triple.split('-')[0]
     return arch in _x86arch
@@ -28,6 +31,32 @@ def dump(header, body):
     print('=' * 80)
 
 
+class _CFG(object):
+    """
+    Wraps the CFG graph for different display method.
+
+    Instance of the class can be stringified (``__repr__`` is defined) to get
+    the graph in DOT format.  The ``.display()`` method plots the graph in
+    PDF.  If in IPython notebook, the returned image can be inlined.
+    """
+    def __init__(self, dot):
+        self.dot = dot
+
+    def display(self, filename=None, view=False):
+        """
+        Plot the CFG.  In IPython notebook, the return image object can be
+        inlined.
+
+        The *filename* option can be set to a specific path for the rendered
+        output to write to.  If *view* option is True, the plot is opened by
+        the system default application for the image format (PDF).
+        """
+        return ll.view_dot_graph(self.dot, filename=filename, view=view)
+
+    def __repr__(self):
+        return self.dot
+
+
 class CodeLibrary(object):
     """
     An interface for bundling LLVM code together and compiling it.
@@ -37,6 +66,7 @@ class CodeLibrary(object):
 
     _finalized = False
     _object_caching_enabled = False
+    _disable_inspection = False
 
     def __init__(self, codegen, name):
         self._codegen = codegen
@@ -45,9 +75,13 @@ class CodeLibrary(object):
         self._final_module = ll.parse_assembly(
             str(self._codegen._create_empty_module(self._name)))
         self._final_module.name = cgutils.normalize_ir_text(self._name)
-        # Remember this on the module, for the object cache hooks
-        self._final_module.__library = weakref.proxy(self)
         self._shared_module = None
+        # Track names of the dynamic globals
+        self._dynamic_globals = []
+
+    @property
+    def has_dynamic_globals(self):
+        return len(self._dynamic_globals) > 0
 
     @property
     def codegen(self):
@@ -87,6 +121,7 @@ class CodeLibrary(object):
         Internal: optimize this library's final module.
         """
         self._codegen._mpm.run(self._final_module)
+        self._final_module = remove_redundant_nrt_refct(self._final_module)
 
     def _get_module_for_linking(self):
         """
@@ -151,7 +186,16 @@ class CodeLibrary(object):
         ll_module.verify()
         self.add_llvm_module(ll_module)
 
+    def _scan_dynamic_globals(self, ll_module):
+        """
+        Scan for dynanmic globals and track their names
+        """
+        for gv in ll_module.global_variables:
+            if gv.name.startswith("numba.dynamic.globals"):
+                self._dynamic_globals.append(gv.name)
+
     def add_llvm_module(self, ll_module):
+        self._scan_dynamic_globals(ll_module)
         self._optimize_functions(ll_module)
         # TODO: we shouldn't need to recreate the LLVM module object
         ll_module = remove_redundant_nrt_refct(ll_module)
@@ -190,6 +234,9 @@ class CodeLibrary(object):
         """
         Make the underlying LLVM module ready to use.
         """
+        # Remember this on the module, for the object cache hooks
+        self._final_module.__library = weakref.proxy(self)
+
         # It seems add_module() must be done only here and not before
         # linking in other modules, otherwise get_pointer_to_function()
         # could fail.
@@ -223,18 +270,34 @@ class CodeLibrary(object):
     def get_function(self, name):
         return self._final_module.get_function(name)
 
+    def _sentry_cache_disable_inspection(self):
+        if self._disable_inspection:
+            warnings.warn('Inspection disabled for cached code. '
+                          'Invalid result is returned.')
+
     def get_llvm_str(self):
         """
         Get the human-readable form of the LLVM module.
         """
+        self._sentry_cache_disable_inspection()
         return str(self._final_module)
 
     def get_asm_str(self):
         """
         Get the human-readable assembly.
         """
+        self._sentry_cache_disable_inspection()
         return str(self._codegen._tm.emit_assembly(self._final_module))
 
+    def get_function_cfg(self, name):
+        """
+        Get control-flow graph of the LLVM function
+        """
+        self._sentry_cache_disable_inspection()
+        fn = self.get_function(name)
+        dot = ll.get_function_cfg(fn)
+        return _CFG(dot)
+
     #
     # Object cache hooks and serialization
     #
@@ -257,6 +320,7 @@ class CodeLibrary(object):
         if self._compiled:
             raise ValueError("library already compiled: %s" % (self,))
         self._compiled_object = value
+        self._disable_inspection = True
 
     @classmethod
     def _dump_elf(cls, buf):
@@ -346,6 +410,8 @@ class CodeLibrary(object):
             self._set_compiled_object(object_code)
             self._shared_module = ll.parse_bitcode(shared_bitcode)
             self._finalize_final_module()
+            # Load symbols from cache
+            self._codegen._engine._load_defined_symbols(self._shared_module)
             return self
         else:
             raise ValueError("unsupported serialization kind %r" % (kind,))
@@ -384,9 +450,20 @@ class JITCodeLibrary(CodeLibrary):
         to the start of the function (as an integer).
 
         This function implicitly calls .finalize().
+
+        Returns
+        -------
+        pointer : int
+            - zero (null) if no symbol of *name* is defined by this code
+              library.
+            - non-zero if the symbol is defined.
         """
         self._ensure_finalized()
-        return self._codegen._engine.get_function_address(name)
+        ee = self._codegen._engine
+        if not ee.is_symbol_defined(name):
+            return 0
+        else:
+            return self._codegen._engine.get_function_address(name)
 
     def _finalize_specific(self):
         self._codegen._scan_and_fix_unresolved_refs(self._final_module)
@@ -400,7 +477,7 @@ class RuntimeLinker(object):
     PREFIX = '.numba.unresolved$'
 
     def __init__(self):
-        self._unresolved = defaultdict(list)
+        self._unresolved = utils.UniqueDict()
         self._defined = set()
         self._resolved = []
 
@@ -414,11 +491,14 @@ class RuntimeLinker(object):
         for gv in module.global_variables:
             if gv.name.startswith(prefix):
                 sym = gv.name[len(prefix):]
+                # Avoid remapping to existing GV
+                if engine.is_symbol_defined(gv.name):
+                    continue
                 # Allocate a memory space for the pointer
-                abortfn = engine.get_function_address('nrt_unresolved_abort')
+                abortfn = rtsys.library.get_pointer_to_function("nrt_unresolved_abort")
                 ptr = ctypes.c_void_p(abortfn)
                 engine.add_global_mapping(gv, ctypes.addressof(ptr))
-                self._unresolved[sym].append(ptr)
+                self._unresolved[sym] = ptr
 
     def scan_defined_symbols(self, module):
         """
@@ -439,15 +519,73 @@ class RuntimeLinker(object):
             # Get runtime address
             fnptr = engine.get_function_address(name)
             # Fix all usage
-            for ptr in self._unresolved[name]:
-                ptr.value = fnptr
-                self._resolved.append((name, ptr))   # keep ptr alive
+            ptr = self._unresolved[name]
+            ptr.value = fnptr
+            self._resolved.append((name, ptr))   # keep ptr alive
             # Delete resolved
             del self._unresolved[name]
 
 
+def _proxy(old):
+    @functools.wraps(old)
+    def wrapper(self, *args, **kwargs):
+        return old(self._ee, *args, **kwargs)
+    return wrapper
+
+
+class JitEngine(object):
+    """Wraps an ExecutionEngine to provide custom symbol tracking.
+    Since the symbol tracking is incomplete  (doesn't consider
+    loaded code object), we are not putting it in llvmlite.
+    """
+    def __init__(self, ee):
+        self._ee = ee
+        # Track symbol defined via codegen'd Module
+        # but not any cached object.
+        # NOTE: `llvm::ExecutionEngine` will catch duplicated symbols and
+        # we are not going to protect against that.  A proper duplicated
+        # symbol detection will need a more logic to check for the linkage
+        # (e.g. like `weak` linkage symbol can override).   This
+        # `_defined_symbols` set will be just enough to tell if a symbol
+        # exists and will not cause the `EE` symbol lookup to `exit(1)`
+        # when symbol-not-found.
+        self._defined_symbols = set()
+
+    def is_symbol_defined(self, name):
+        """Is the symbol defined in this session?
+        """
+        return name in self._defined_symbols
+
+    def _load_defined_symbols(self, mod):
+        """Extract symbols from the module
+        """
+        for gsets in (mod.functions, mod.global_variables):
+            self._defined_symbols |= {gv.name for gv in gsets
+                                      if not gv.is_declaration}
+
+    def add_module(self, module):
+        """Override ExecutionEngine.add_module
+        to keep info about defined symbols.
+        """
+        self._load_defined_symbols(module)
+        return self._ee.add_module(module)
+
+    def add_global_mapping(self, gv, addr):
+        """Override ExecutionEngine.add_global_mapping
+        to keep info about defined symbols.
+        """
+        self._defined_symbols.add(gv.name)
+        return self._ee.add_global_mapping(gv, addr)
+
+    #
+    # The remaining methods are re-export of the ExecutionEngine APIs
+    #
+    set_object_cache = _proxy(ll.ExecutionEngine.set_object_cache)
+    finalize_object = _proxy(ll.ExecutionEngine.finalize_object)
+    get_function_address = _proxy(ll.ExecutionEngine.get_function_address)
+
+
 class BaseCPUCodegen(object):
-    _llvm_initialized = False
 
     def __init__(self, module_name):
         initialize_llvm()
@@ -471,7 +609,7 @@ class BaseCPUCodegen(object):
         engine = ll.create_mcjit_compiler(llvm_module, tm)
 
         self._tm = tm
-        self._engine = engine
+        self._engine = JitEngine(engine)
         self._target_data = engine.target_data
         self._data_layout = str(self._target_data)
         self._mpm = self._module_pass_manager()
@@ -513,8 +651,6 @@ class BaseCPUCodegen(object):
 
     def _module_pass_manager(self):
         pm = ll.create_module_pass_manager()
-        dl = ll.create_target_data(self._data_layout)
-        dl.add_pass(pm)
         self._tm.add_analysis_passes(pm)
         with self._pass_manager_builder() as pmb:
             pmb.populate(pm)
@@ -522,7 +658,6 @@ class BaseCPUCodegen(object):
 
     def _function_pass_manager(self, llvm_module):
         pm = ll.create_function_pass_manager(llvm_module)
-        self._target_data.add_pass(pm)
         self._tm.add_analysis_passes(pm)
         with self._pass_manager_builder() as pmb:
             pmb.populate(pm)
@@ -575,7 +710,7 @@ class BaseCPUCodegen(object):
         """
         Return a tuple unambiguously describing the codegen behaviour.
         """
-        return (self._llvm_module.triple, ll.get_host_cpu_name(),
+        return (self._llvm_module.triple, self._get_host_cpu_name(),
                 self._tm_features)
 
     def _scan_and_fix_unresolved_refs(self, module):
@@ -587,13 +722,23 @@ class BaseCPUCodegen(object):
         voidptr = llvmir.IntType(8).as_pointer()
         ptrname = self._rtlinker.PREFIX + name
         llvm_mod = builder.module
-        fnptr = llvm_mod.get_global(ptrname)
-        # Not defined?
-        if fnptr is None:
+        try:
+            fnptr = llvm_mod.get_global(ptrname)
+        except KeyError:
+            # Not defined?
             fnptr = llvmir.GlobalVariable(llvm_mod, voidptr, name=ptrname)
             fnptr.linkage = 'external'
         return builder.bitcast(builder.load(fnptr), fnty.as_pointer())
 
+    def _get_host_cpu_name(self):
+        return (ll.get_host_cpu_name()
+                if config.CPU_NAME is None
+                else config.CPU_NAME)
+
+    def _get_host_cpu_features(self):
+        if config.CPU_FEATURES is not None:
+            return config.CPU_FEATURES
+        return get_host_cpu_features()
 
 class AOTCPUCodegen(BaseCPUCodegen):
     """
@@ -611,7 +756,7 @@ class AOTCPUCodegen(BaseCPUCodegen):
     def _customize_tm_options(self, options):
         cpu_name = self._cpu_name
         if cpu_name == 'host':
-            cpu_name = ll.get_host_cpu_name()
+            cpu_name = self._get_host_cpu_name()
         options['cpu'] = cpu_name
         options['reloc'] = 'pic'
         options['codemodel'] = 'default'
@@ -636,8 +781,7 @@ class JITCPUCodegen(BaseCPUCodegen):
     def _customize_tm_options(self, options):
         # As long as we don't want to ship the code to another machine,
         # we can specialize for this CPU.
-        options['cpu'] = ll.get_host_cpu_name()
-
+        options['cpu'] = self._get_host_cpu_name()
         options['reloc'] = 'default'
         options['codemodel'] = 'jitdefault'
 
@@ -650,8 +794,36 @@ class JITCPUCodegen(BaseCPUCodegen):
 
     def _customize_tm_features(self):
         # For JIT target, we will use LLVM to get the feature map
-        features = ll.get_host_cpu_features()
+        return self._get_host_cpu_features()
 
+    def _add_module(self, module):
+        self._engine.add_module(module)
+        # XXX: disabling remove module due to MCJIT engine leakage in
+        #      removeModule.  The removeModule causes consistent access
+        #      violation with certain test combinations.
+        # # Early bind the engine method to avoid keeping a reference to self.
+        # return functools.partial(self._engine.remove_module, module)
+
+
+def initialize_llvm():
+    """Safe to use multiple times.
+    """
+    ll.initialize()
+    ll.initialize_native_target()
+    ll.initialize_native_asmprinter()
+
+
+def get_host_cpu_features():
+    """Get host CPU features using LLVM.
+
+    The features may be modified due to user setting.
+    See numba.config.ENABLE_AVX.
+    """
+    try:
+        features = ll.get_host_cpu_features()
+    except RuntimeError:
+        return ''
+    else:
         if not config.ENABLE_AVX:
             # Disable all features with name starting with 'avx'
             for k in features:
@@ -660,19 +832,3 @@ class JITCPUCodegen(BaseCPUCodegen):
 
         # Set feature attributes
         return features.flatten()
-
-    def _add_module(self, module):
-        self._engine.add_module(module)
-        # Early bind the engine method to avoid keeping a reference to self.
-        return functools.partial(self._engine.remove_module, module)
-
-
-_llvm_initialized = False
-
-def initialize_llvm():
-    global _llvm_initialized
-    if not _llvm_initialized:
-        ll.initialize()
-        ll.initialize_native_target()
-        ll.initialize_native_asmprinter()
-        _llvm_initialized = True
