@@ -20,6 +20,8 @@ from ..config import PYVERSION
 from ..numpy_support import ufunc_find_matching_loop, select_array_wrapper
 from ..typing import npydecl
 
+from .. import errors
+
 registry = Registry()
 lower = registry.lower
 
@@ -166,7 +168,7 @@ def _prepare_argument(ctxt, bld, inp, tyinp, where='input operand'):
     elif tyinp in types.number_domain | set([types.boolean]):
         return _ScalarHelper(ctxt, bld, inp, tyinp)
     else:
-        raise TypeError('unknown type for {0}: {1}'.format(where, str(tyinp)))
+        raise NotImplementedError('unsupported type for {0}: {1}'.format(where, str(tyinp)))
 
 
 _broadcast_onto_sig = types.intp(types.intp, types.CPointer(types.intp),
@@ -252,6 +254,11 @@ def _build_array(context, builder, array_ty, input_types, inputs):
                                  builder.icmp(lc.ICMP_SLT, arg_result, ONE)):
             msg = "unable to broadcast argument %d to output array" % (
                 arg_number,)
+
+            loc = errors.loc_info.get('loc', None)
+            if loc is not None:
+                msg += '\nFile "%s", line %d, ' % (loc.filename, loc.line)
+
             context.call_conv.return_user_exc(builder, ValueError, (msg,))
 
     real_array_ty = array_ty.as_array
@@ -313,7 +320,7 @@ def numpy_ufunc_kernel(context, builder, sig, args, kernel_class,
         arguments.append(output)
     elif context.enable_nrt:
         # Incref the output
-        context.nrt_incref(builder, sig.return_type, args[-1])
+        context.nrt.incref(builder, sig.return_type, args[-1])
 
     inputs = arguments[0:-1]
     output = arguments[-1]
