@@ -494,6 +494,23 @@ lower_builtin('<=', types.boolean, types.boolean)(int_ule_impl)
 lower_builtin('>', types.boolean, types.boolean)(int_ugt_impl)
 lower_builtin('>=', types.boolean, types.boolean)(int_uge_impl)
 
+@lower_builtin('==', types.Const, types.Const)
+def const_eq_impl(context, builder, sig, args):
+    arg1, arg2 = sig.args
+    val = 0
+    if arg1.value==arg2.value:
+        val = 1
+    res = ir.Constant(ir.IntType(1), val)
+    return impl_ret_untracked(context, builder, sig.return_type, res)
+
+@lower_builtin('!=', types.Const, types.Const)
+def const_eq_impl(context, builder, sig, args):
+    arg1, arg2 = sig.args
+    val = 0
+    if arg1.value!=arg2.value:
+        val = 1
+    res = ir.Constant(ir.IntType(1), val)
+    return impl_ret_untracked(context, builder, sig.return_type, res)
 
 def _implement_integer_operators():
     ty = types.Integer
@@ -571,7 +588,7 @@ def real_divmod(context, builder, x, y):
     floatty = x.type
 
     module = builder.module
-    fname = ".numba.python.rem.%s" % x.type
+    fname = context.mangler(".numba.python.rem", [x.type])
     fnty = Type.function(floatty, (floatty, floatty, Type.pointer(floatty)))
     fn = module.get_or_insert_function(fnty, fname)
 
@@ -674,9 +691,12 @@ def real_divmod_func_body(context, builder, vx, wx):
     div_istrue = builder.fcmp(lc.FCMP_ONE, div, ZERO)
 
     with builder.if_then(div_istrue):
-        module = builder.module
-        floorfn = lc.Function.intrinsic(module, lc.INTR_FLOOR, [wx.type])
-        floordiv = builder.call(floorfn, [div])
+        realtypemap = {'float': types.float32,
+                       'double': types.float64}
+        realtype = realtypemap[str(wx.type)]
+        floorfn = context.get_function(math.floor,
+                                       typing.signature(realtype, realtype))
+        floordiv = floorfn(builder, [div])
         floordivdiff = builder.fsub(div, floordiv)
         floordivincr = builder.fadd(floordiv, ONE)
         HALF = Constant.real(wx.type, 0.5)
@@ -1239,6 +1259,24 @@ def hash_complex(context, builder, sig, args):
 #-------------------------------------------------------------------------------
 # Implicit casts between numerics
 
+@lower_cast(types.Integer, types.Const)
+def integer_to_constant(context, builder, fromty, toty, val):
+    # Perform runtime check to ensure that the runtime value
+    # matches the expected constant.
+    # The violation would imply an internal error.
+    # The runtime checking logic cannot be tested automatically.
+    # The easiest way to test is to change the comparison from `!=` to `==`
+    # so that the exception will raise when the expection is met.
+    const = context.get_constant(fromty, toty.value)
+    matches = builder.icmp_unsigned('!=', val, const)
+    with cgutils.if_unlikely(builder, matches):
+        # Raise RuntimeError about the assumption violation
+        usermsg = "numba constant integer assumption violated"
+        errmsg = "{}: expecting {}".format(usermsg, toty.value)
+        context.call_conv.return_user_exc(builder, RuntimeError, (errmsg,))
+    return cgutils.get_null_value(context.get_value_type(toty))
+
+
 @lower_cast(types.Integer, types.Integer)
 def integer_to_integer(context, builder, fromty, toty, val):
     if toty.bitwidth == fromty.bitwidth:
