@@ -1,6 +1,6 @@
 from __future__ import division
 
-from itertools import product
+from itertools import product, cycle
 import sys
 
 import numpy as np
@@ -8,6 +8,7 @@ import numpy as np
 from numba import unittest_support as unittest
 from numba import jit, typeof, types
 from numba.compiler import compile_isolated
+from numba.errors import TypingError, LoweringError
 from numba.numpy_support import (as_dtype, strict_ufunc_typing,
                                  version as numpy_version)
 from .support import TestCase, CompilationCache, MemoryLeak, MemoryLeakMixin, tag
@@ -72,9 +73,21 @@ def np_copy(arr):
 def np_asfortranarray(arr):
     return np.asfortranarray(arr)
 
+def np_ascontiguousarray(arr):
+    return np.ascontiguousarray(arr)
+
 def array_view(arr, newtype):
     return arr.view(newtype)
 
+def array_take(arr, indices):
+    return arr.take(indices)
+
+def array_take_kws(arr, indices, axis):
+    return arr.take(indices, axis=axis)
+
+def array_fill(arr, val):
+    return arr.fill(val)
+
 # XXX Can't pass a dtype as a Dispatcher argument for now
 def make_array_view(newtype):
     def array_view(arr):
@@ -131,6 +144,32 @@ def array_item(a):
 def array_itemset(a, v):
     a.itemset(v)
 
+def array_sum(a, *args):
+    return a.sum(*args)
+
+def array_sum_kws(a, axis):
+    return a.sum(axis=axis)
+
+def array_sum_const_multi(arr, axis):
+    # use np.sum with different constant args multiple times to check
+    # for internal compile cache to see if constant-specialization is
+    # applied properly.
+    a = np.sum(arr, axis=4)
+    b = np.sum(arr, 3)
+    # the last invocation uses runtime-variable
+    c = np.sum(arr, axis)
+    # as method
+    d = arr.sum(axis=5)
+    # negative const axis
+    e = np.sum(arr, axis=-1)
+    return a, b, c, d, e
+
+def array_cumsum(a, *args):
+    return a.cumsum(*args)
+
+def array_cumsum_kws(a, axis):
+    return a.cumsum(axis=axis)
+
 
 class TestArrayMethods(MemoryLeakMixin, TestCase):
     """
@@ -435,6 +474,10 @@ class TestArrayMethods(MemoryLeakMixin, TestCase):
         self.check_layout_dependent_func(np_asfortranarray,
                                          check_sameness=numpy_version >= (1, 8))
 
+    def test_np_ascontiguousarray(self):
+        self.check_layout_dependent_func(np_ascontiguousarray,
+                                         check_sameness=numpy_version > (1, 11))
+
     def check_np_frombuffer_allocated(self, pyfunc):
         def run(shape):
             cres = self.ccache.compile(pyfunc, (typeof(shape),))
@@ -602,6 +645,187 @@ class TestArrayMethods(MemoryLeakMixin, TestCase):
         check_err(np.array([1, 2]))
         check_err(np.array([]))
 
+    def test_sum(self):
+        pyfunc = array_sum
+        cfunc = jit(nopython=True)(pyfunc)
+        # OK
+        a = np.ones((7, 6, 5, 4, 3))
+        self.assertPreciseEqual(pyfunc(a), cfunc(a))
+        # OK
+        self.assertPreciseEqual(pyfunc(a, 0), cfunc(a, 0))
+
+    def test_sum_kws(self):
+        pyfunc = array_sum_kws
+        cfunc = jit(nopython=True)(pyfunc)
+        # OK
+        a = np.ones((7, 6, 5, 4, 3))
+        self.assertPreciseEqual(pyfunc(a, axis=1), cfunc(a, axis=1))
+        # OK
+        self.assertPreciseEqual(pyfunc(a, axis=2), cfunc(a, axis=2))
+
+    def test_sum_const(self):
+        pyfunc = array_sum_const_multi
+        cfunc = jit(nopython=True)(pyfunc)
+
+        arr = np.ones((3, 4, 5, 6, 7, 8))
+        axis = 1
+        self.assertPreciseEqual(pyfunc(arr, axis), cfunc(arr, axis))
+        axis = 2
+        self.assertPreciseEqual(pyfunc(arr, axis), cfunc(arr, axis))
+
+    def test_sum_exceptions(self):
+        # Exceptions leak references
+        self.disable_leak_check()
+        pyfunc = array_sum
+        cfunc = jit(nopython=True)(pyfunc)
+
+        a = np.ones((7, 6, 5, 4, 3))
+        b = np.ones((4, 3))
+        # BAD: axis > dimensions
+        with self.assertRaises(ValueError):
+            cfunc(b, 2)
+        # BAD: negative axis
+        with self.assertRaises(ValueError):
+            cfunc(a, -1)
+        # BAD: axis greater than 3
+        with self.assertRaises(ValueError):
+            cfunc(a, 4)
+
+    def test_sum_const_negative(self):
+        # Exceptions leak references
+        self.disable_leak_check()
+
+        @jit(nopython=True)
+        def foo(arr):
+            return arr.sum(axis=-3)
+
+        # ndim == 4, axis == -3, OK
+        a = np.ones((1, 2, 3, 4))
+        self.assertPreciseEqual(foo(a), foo.py_func(a))
+        # ndim == 3, axis == -3, OK
+        a = np.ones((1, 2, 3))
+        self.assertPreciseEqual(foo(a), foo.py_func(a))
+        # ndim == 2, axis == -3, BAD
+        a = np.ones((1, 2))
+        with self.assertRaises(LoweringError) as raises:
+            foo(a)
+        errmsg = "'axis' entry is out of bounds"
+        self.assertIn(errmsg, str(raises.exception))
+        with self.assertRaises(ValueError) as raises:
+            foo.py_func(a)
+        # Numpy 1.13 has a different error message than prior numpy
+        # Just check for the "out of bounds" phrase in it.
+        self.assertIn("out of bounds", str(raises.exception))
+
+    def test_cumsum(self):
+        pyfunc = array_cumsum
+        cfunc = jit(nopython=True)(pyfunc)
+        # OK
+        a = np.ones((2, 3))
+        self.assertPreciseEqual(pyfunc(a), cfunc(a))
+        # BAD: with axis
+        with self.assertRaises(TypingError):
+            cfunc(a, 1)
+        # BAD: with kw axis
+        pyfunc = array_cumsum_kws
+        cfunc = jit(nopython=True)(pyfunc)
+        with self.assertRaises(TypingError):
+            cfunc(a, axis=1)
+
+
+    def test_take(self):
+        pyfunc = array_take
+        cfunc = jit(nopython=True)(pyfunc)
+
+        def check(arr, ind):
+            expected = pyfunc(arr, ind)
+            got = cfunc(arr, ind)
+            self.assertPreciseEqual(expected, got)
+            if hasattr(expected, 'order'):
+                self.assertEqual(expected.order == got.order)
+
+        # need to check:
+        # 1. scalar index
+        # 2. 1d array index
+        # 3. nd array index, >2d and F order
+        # 4. reflected list
+        # 5. tuples
+
+        test_indices = []
+        test_indices.append(1)
+        test_indices.append(5)
+        test_indices.append(11)
+        test_indices.append(-2)
+        test_indices.append(np.array([1, 5, 1, 11, 3]))
+        test_indices.append(np.array([[1, 5, 1], [11, 3, 0]], order='F'))
+        test_indices.append(np.array([[[1, 5, 1], [11, 3, 0]]]))
+        test_indices.append(np.array([[[[1, 5]], [[11, 0]],[[1, 2]]]]))
+        test_indices.append([1, 5, 1, 11, 3])
+        test_indices.append((1, 5, 1))
+        test_indices.append(((1, 5, 1), (11, 3, 2)))
+        test_indices.append((((1,), (5,), (1,)), ((11,), (3,), (2,))))
+
+        layouts = cycle(['C', 'F', 'A'])
+
+        for dt in [np.float64, np.int64, np.complex128]:
+            A = np.arange(12, dtype=dt).reshape((4, 3), order=next(layouts))
+            for ind in test_indices:
+                check(A, ind)
+
+        #check illegal access raises
+        A = np.arange(12, dtype=dt).reshape((4, 3), order=next(layouts))
+        szA = A.size
+        illegal_indices = [szA, -szA - 1, np.array(szA), np.array(-szA - 1),
+                           [szA], [-szA - 1]]
+        for x in illegal_indices:
+            with self.assertRaises(IndexError):
+                cfunc(A, x) # oob raises
+
+        # check float indexing raises
+        with self.assertRaises(TypingError):
+            cfunc(A, [1.7])
+
+        # check unsupported arg raises
+        with self.assertRaises(TypingError):
+            take_kws = jit(nopython=True)(array_take_kws)
+            take_kws(A, 1, 1)
+
+        # check kwarg unsupported raises
+        with self.assertRaises(TypingError):
+            take_kws = jit(nopython=True)(array_take_kws)
+            take_kws(A, 1, axis=1)
+
+        #exceptions leak refs
+        self.disable_leak_check()
+
+
+    def test_fill(self):
+        pyfunc = array_fill
+        cfunc = jit(nopython=True)(pyfunc)
+        def check(arr, val):
+            expected = np.copy(arr)
+            erv = pyfunc(expected, val)
+            self.assertTrue(erv is None)
+            got = np.copy(arr)
+            grv = cfunc(got, val)
+            self.assertTrue(grv is None)
+            # check mutation is the same
+            self.assertPreciseEqual(expected, got)
+
+        # scalar
+        A = np.arange(1)
+        for x in [np.float64, np.bool_]:
+            check(A, x(10))
+
+        # 2d
+        A = np.arange(12).reshape(3, 4)
+        for x in [np.float64, np.bool_]:
+            check(A, x(10))
+
+        # 4d
+        A = np.arange(48, dtype=np.complex64).reshape(2, 3, 4, 2)
+        for x in [np.float64, np.complex128, np.bool_]:
+            check(A, x(10))
 
 class TestArrayComparisons(TestCase):
 
