@@ -104,6 +104,9 @@ def array_nanmean(arr):
 def array_nansum(arr):
     return np.nansum(arr)
 
+def array_nanprod(arr):
+    return np.nanprod(arr)
+
 def array_nanstd(arr):
     return np.nanstd(arr)
 
@@ -271,6 +274,11 @@ class TestArrayReductions(MemoryLeakMixin, TestCase):
         self.check_reduction_basic(array_nansum,
                                    all_nans=np_version >= (1, 9))
 
+    @tag('important')
+    @unittest.skipUnless(np_version >= (1, 10), "nanprod needs Numpy 1.10+")
+    def test_nanprod_basic(self):
+        self.check_reduction_basic(array_nanprod)
+
     @tag('important')
     @unittest.skipUnless(np_version >= (1, 8), "nanstd needs Numpy 1.8+")
     def test_nanstd_basic(self):
@@ -337,6 +345,8 @@ class TestArrayReductions(MemoryLeakMixin, TestCase):
             yield a
             a[a % 4 >= 2] = float('nan')
             yield a
+            a[:] = float('nan')
+            yield a
 
         self.check_median_basic(pyfunc, variations)
 
@@ -531,6 +541,8 @@ class TestArrayReductions(MemoryLeakMixin, TestCase):
                            ]
         if np_version >= (1, 8):
             reduction_funcs += [array_nanmean, array_nanstd, array_nanvar]
+        if np_version >= (1, 10):
+            reduction_funcs += [array_nanprod]
 
         dtypes_to_test = [np.int32, np.float32, np.bool_]
 
