@@ -7,6 +7,7 @@ import random
 import threading
 
 from numba import unittest_support as unittest
+from numba.errors import TypingError
 from numba import njit
 from numba import utils
 from numba.numpy_support import version as numpy_version
@@ -701,6 +702,12 @@ class ConstructorLikeBaseTest(object):
             # Non-contiguous array
             if arr.ndim > 0:
                 check_arr(arr[::2])
+            # Check new array doesn't inherit readonly flag
+            arr.flags['WRITEABLE'] = False
+            # verify read-only
+            with self.assertRaises(ValueError):
+                arr[0] = 1
+            check_arr(arr)
 
         # Scalar argument => should produce a 0-d array
         check_arr(orig[0])
@@ -931,7 +938,6 @@ class TestNdDiag(TestCase):
 
     # check error handling
     def test_error_handling(self):
-        from numba.errors import TypingError
         d = np.array([[[1.]]])
         cfunc = nrtjit(self.py)
 
@@ -1001,7 +1007,6 @@ class TestNpyEmptyKeyword(TestCase):
             self._test_with_shape_and_dtype_kw(dtype)
 
     def test_empty_no_args(self):
-        from numba.errors import TypingError
 
         def pyfunc():
             return np.empty()
@@ -1088,6 +1093,33 @@ class TestNpArray(MemoryLeakMixin, BaseTest):
         got = cfunc([(1, 2.5), (3, 4.5)])
         self.assertPreciseEqual(got, np.int32([[1, 2], [3, 4]]))
 
+    def test_raises(self):
+
+        def pyfunc(arg):
+            return np.array(arg)
+
+        cfunc = nrtjit(pyfunc)
+
+        @contextlib.contextmanager
+        def check_raises(msg):
+            with self.assertRaises(TypingError) as raises:
+                yield
+            self.assertIn(msg, str(raises.exception))
+
+        with check_raises(('array(float64, 1d, C) not allowed in a '
+                           'homogenous sequence')):
+            cfunc(np.array([1.]))
+
+        with check_raises(('type (int64, reflected list(int64)) does '
+                          'not have a regular shape')):
+            cfunc((np.int64(1), [np.int64(2)]))
+
+        with check_raises(("cannot convert (int64, Record([('a', '<i4'), "
+                           "('b', '<f4')])) to a homogenous type")):
+            st = np.dtype([('a', 'i4'), ('b', 'f4')])
+            val = np.zeros(1, dtype=st)[0]
+            cfunc(((1, 2), (np.int64(1), val)))
+
 
 class TestNpConcatenate(MemoryLeakMixin, TestCase):
     """
