@@ -220,6 +220,13 @@ class TestJitClass(TestCase, MemoryLeakMixin):
                 # use deferred type as argument
                 return get_data(self.next)
 
+            def append_to_tail(self, other):
+                cur = self
+                while cur.next is not None:
+                    cur = cur.next
+                cur.next = other
+
+
         node_type.define(LinkedNode.class_type.instance_type)
 
         first = LinkedNode(123, None)
@@ -239,6 +246,16 @@ class TestJitClass(TestCase, MemoryLeakMixin):
         first_val = second.get_next_data()
         self.assertEqual(first_val, first.data)
 
+        # Check setattr (issue #2606)
+        self.assertIsNone(first.next)
+        second.append_to_tail(LinkedNode(567, None))
+        self.assertIsNotNone(first.next)
+        self.assertEqual(first.next.data, 567)
+        self.assertIsNone(first.next.next)
+        second.append_to_tail(LinkedNode(678, None))
+        self.assertIsNotNone(first.next.next)
+        self.assertEqual(first.next.next.data, 678)
+
         # Check ownership
         self.assertEqual(first_meminfo.refcount, 3)
         del second, second_meminfo
@@ -584,6 +601,24 @@ class TestJitClass(TestCase, MemoryLeakMixin):
         self.assertEqual(Apple.foo.__doc__, 'foo method docstring')
         self.assertEqual(Apple.aval.__doc__, 'aval property docstring')
 
+    def test_kwargs(self):
+        spec = [('a', int32),
+                ('b', float64)]
+
+        @jitclass(spec)
+        class TestClass(object):
+            def __init__(self, x, y, z):
+                self.a = x * y
+                self.b = z
+
+        x = 2
+        y = 2
+        z = 1.1
+        kwargs = {'y': y, 'z': z}
+        tc = TestClass(x=2, **kwargs)
+        self.assertEqual(tc.a, x * y)
+        self.assertEqual(tc.b, z)
+
 
 if __name__ == '__main__':
     unittest.main()
