@@ -1,24 +1,35 @@
 from __future__ import division, print_function
 
-from numba import unittest_support as unittest
-
 import sys
 import subprocess
 
+from numba import unittest_support as unittest
+from numba import cuda
+
 
 class TestCase(unittest.TestCase):
     """These test cases are meant to test the Numba test infrastructure itself.
     Therefore, the logic used here shouldn't use numba.testing, but only the upstream
     unittest, and run the numba test suite only in a subprocess."""
 
+    def get_testsuite_listing(self, args):
+        cmd = ['python', '-m', 'numba.runtests', '-l'] + list(args)
+        lines = subprocess.check_output(cmd).decode().splitlines()
+        lines = [line for line in lines if line.strip()]
+        return lines
+
+    def check_listing_prefix(self, prefix):
+        listing = self.get_testsuite_listing([prefix])
+        for ln in listing[:-1]:
+            errmsg = '{!r} not startswith {!r}'.format(ln, prefix)
+            self.assertTrue(ln.startswith(prefix), msg=errmsg)
+
     def check_testsuite_size(self, args, minsize, maxsize=None):
         """
         Check that the reported numbers of tests are in the
         (minsize, maxsize) range, or are equal to minsize if maxsize is None.
         """
-        cmd = ['python', '-m', 'numba.runtests', '-l'] + list(args)
-        lines = subprocess.check_output(cmd).decode().splitlines()
-        lines = [line for line in lines if line.strip()]
+        lines = self.get_testsuite_listing(args)
         last_line = lines[-1]
         self.assertTrue(last_line.endswith('tests found'))
         number = int(last_line.split(' ')[0])
@@ -48,14 +59,21 @@ class TestCase(unittest.TestCase):
     def test_cuda(self):
         # Even without CUDA enabled, there is at least one test
         # (in numba.cuda.tests.nocuda)
-        self.check_testsuite_size(['numba.cuda.tests'], 1, 400)
+        self.check_testsuite_size(['numba.cuda.tests'], 1, 450)
+
+    @unittest.skipIf(not cuda.is_available(), "NO CUDA")
+    def test_cuda_submodules(self):
+        self.check_listing_prefix('numba.cuda.tests.cudadrv')
+        self.check_listing_prefix('numba.cuda.tests.cudapy')
+        self.check_listing_prefix('numba.cuda.tests.nocuda')
+        self.check_listing_prefix('numba.cuda.tests.cudasim')
 
     def test_module(self):
-        self.check_testsuite_size(['numba.tests.test_utils'], 5, 15)
+        self.check_testsuite_size(['numba.tests.test_utils'], 3, 15)
         self.check_testsuite_size(['numba.tests.test_nested_calls'], 5, 15)
         # Several modules
         self.check_testsuite_size(['numba.tests.test_nested_calls',
-                                   'numba.tests.test_utils'], 15, 30)
+                                   'numba.tests.test_utils'], 13, 30)
 
     def test_subpackage(self):
         self.check_testsuite_size(['numba.tests.npyufunc'], 50, 200)
