@@ -5,22 +5,24 @@ import itertools
 import re
 import sys
 import warnings
+import threading
 
 import numpy as np
 
 import numba.unittest_support as unittest
 from numba import types, typing, utils, typeof, numpy_support, njit
 from numba.compiler import compile_isolated, Flags, DEFAULT_FLAGS
-from numba.numpy_support import from_dtype
+from numba.numpy_support import from_dtype, version as numpy_version
 from numba import jit, vectorize
 from numba.config import PYVERSION
 from numba.errors import LoweringError, TypingError
 from .support import TestCase, CompilationCache, MemoryLeakMixin, tag
-
+from numba.six import exec_
 from numba.typing.npydecl import supported_ufuncs, all_ufuncs
 
 is32bits = tuple.__itemsize__ == 4
 iswindows = sys.platform.startswith('win32')
+after_numpy_112 = numpy_version >= (1, 12)
 
 # NOTE: to test the implementation of Numpy ufuncs, we disable rewriting
 # of array expressions.
@@ -52,7 +54,7 @@ def _make_ufunc_usecase(ufunc):
     ldict = {}
     arg_str = ','.join(['a{0}'.format(i) for i in range(ufunc.nargs)])
     func_str = 'def fn({0}):\n    np.{1}({0})'.format(arg_str, ufunc.__name__)
-    exec(func_str, globals(), ldict)
+    exec_(func_str, globals(), ldict)
     fn = ldict['fn']
     fn.__name__ = '{0}_usecase'.format(ufunc.__name__)
     return fn
@@ -61,7 +63,7 @@ def _make_ufunc_usecase(ufunc):
 def _make_unary_ufunc_usecase(ufunc):
     ufunc_name = ufunc.__name__
     ldict = {}
-    exec("def fn(x,out):\n    np.{0}(x,out)".format(ufunc_name), globals(), ldict)
+    exec_("def fn(x,out):\n    np.{0}(x,out)".format(ufunc_name), globals(), ldict)
     fn = ldict["fn"]
     fn.__name__ = "{0}_usecase".format(ufunc_name)
     return fn
@@ -69,7 +71,7 @@ def _make_unary_ufunc_usecase(ufunc):
 
 def _make_unary_ufunc_op_usecase(ufunc_op):
     ldict = {}
-    exec("def fn(x):\n    return {0}(x)".format(ufunc_op), globals(), ldict)
+    exec_("def fn(x):\n    return {0}(x)".format(ufunc_op), globals(), ldict)
     fn = ldict["fn"]
     fn.__name__ = "usecase_{0}".format(hash(ufunc_op))
     return fn
@@ -78,7 +80,7 @@ def _make_unary_ufunc_op_usecase(ufunc_op):
 def _make_binary_ufunc_usecase(ufunc):
     ufunc_name = ufunc.__name__
     ldict = {}
-    exec("def fn(x,y,out):\n    np.{0}(x,y,out)".format(ufunc_name), globals(), ldict);
+    exec_("def fn(x,y,out):\n    np.{0}(x,y,out)".format(ufunc_name), globals(), ldict);
     fn = ldict['fn']
     fn.__name__ = "{0}_usecase".format(ufunc_name)
     return fn
@@ -86,7 +88,7 @@ def _make_binary_ufunc_usecase(ufunc):
 
 def _make_binary_ufunc_op_usecase(ufunc_op):
     ldict = {}
-    exec("def fn(x,y):\n    return x{0}y".format(ufunc_op), globals(), ldict)
+    exec_("def fn(x,y):\n    return x{0}y".format(ufunc_op), globals(), ldict)
     fn = ldict["fn"]
     fn.__name__ = "usecase_{0}".format(hash(ufunc_op))
     return fn
@@ -94,7 +96,7 @@ def _make_binary_ufunc_op_usecase(ufunc_op):
 
 def _make_inplace_ufunc_op_usecase(ufunc_op):
     ldict = {}
-    exec("def fn(x,y):\n    x{0}y".format(ufunc_op), globals(), ldict)
+    exec_("def fn(x,y):\n    x{0}y".format(ufunc_op), globals(), ldict)
     fn = ldict["fn"]
     fn.__name__ = "usecase_{0}".format(hash(ufunc_op))
     return fn
@@ -247,7 +249,7 @@ class TestUFuncs(BaseUFuncTest, TestCase):
     def binary_ufunc_test(self, ufunc, flags=no_pyobj_flags,
                          skip_inputs=[], additional_inputs=[],
                          int_output_type=None, float_output_type=None,
-                         kinds='ifc'):
+                         kinds='ifc', positive_only=False):
 
         ufunc = _make_binary_ufunc_usecase(ufunc)
 
@@ -260,6 +262,8 @@ class TestUFuncs(BaseUFuncTest, TestCase):
 
             if input_type in skip_inputs:
                 continue
+            if positive_only and np.any(input_operand < 0):
+                continue
 
             # Some ufuncs don't allow all kinds of arguments, and implicit
             # conversion has become stricter in 1.10.
@@ -348,7 +352,8 @@ class TestUFuncs(BaseUFuncTest, TestCase):
 
     @tag('important')
     def test_power_ufunc(self, flags=no_pyobj_flags):
-        self.binary_ufunc_test(np.power, flags=flags)
+        self.binary_ufunc_test(np.power, flags=flags,
+                               positive_only=after_numpy_112)
 
     @tag('important')
     def test_remainder_ufunc(self, flags=no_pyobj_flags):
@@ -1184,7 +1189,7 @@ class TestArrayOperators(BaseUFuncTest, TestCase):
 
     @tag('important')
     def test_power_array_op(self):
-        self.binary_op_test('**')
+        self.binary_op_test('**', positive_rhs=after_numpy_112)
 
     @tag('important')
     def test_left_shift_array_op(self):
@@ -1526,12 +1531,26 @@ class TestLoopTypesIntNoPython(_LoopTypesTester):
     _ufuncs.remove(np.reciprocal)
     _ufuncs.remove(np.left_shift) # has its own test class
     _ufuncs.remove(np.right_shift) # has its own test class
+    # special test for bool subtract/negative, np1.13 deprecated/not supported
+    _ufuncs.remove(np.subtract)
+    _ufuncs.remove(np.negative)
     _required_types = '?bBhHiIlLqQ'
     _skip_types = 'fdFDmMO' + _LoopTypesTester._skip_types
 
 TestLoopTypesIntNoPython.autogenerate()
 
 
+class TestLoopTypesSubtractAndNegativeNoPython(_LoopTypesTester):
+    _compile_flags = no_pyobj_flags
+    _ufuncs = [np.subtract, np.negative]
+    _required_types = '?bBhHiIlLqQfdFD'
+    _skip_types = 'mMO' + _LoopTypesTester._skip_types
+    if after_numpy_112: # np1.13 deprecated/not supported
+        _skip_types += '?'
+
+TestLoopTypesSubtractAndNegativeNoPython.autogenerate()
+
+
 class TestLoopTypesReciprocalNoPython(_LoopTypesTester):
     _compile_flags = no_pyobj_flags
     _ufuncs = [np.reciprocal] # issue #757
@@ -1802,6 +1821,34 @@ class TestUFuncBadArgsNoPython(TestCase):
         self.assertRaises(TypingError, compile_isolated, func, [types.float64],
                           return_type=types.float64, flags=self._compile_flags)
 
+class TestUFuncCompilationThreadSafety(TestCase):
+
+    def test_lock(self):
+        """
+        Test that (lazy) compiling from several threads at once doesn't
+        produce errors (see issue #2403).
+        """
+        errors = []
+
+        @vectorize
+        def foo(x):
+            return x + 1
+
+        def wrapper():
+            try:
+                a = np.ones((10,), dtype = np.float64)
+                expected = np.ones((10,), dtype = np.float64) + 1.
+                np.testing.assert_array_equal(foo(a), expected)
+            except BaseException as e:
+                errors.append(e)
+
+        threads = [threading.Thread(target=wrapper) for i in range(16)]
+        for t in threads:
+            t.start()
+        for t in threads:
+            t.join()
+        self.assertFalse(errors)
+
 
 if __name__ == '__main__':
     unittest.main()
