@@ -93,6 +93,17 @@ class Type(object):
         """
         return self.name
 
+    @property
+    def mangling_args(self):
+        """
+        Returns `(basename, args)` where `basename` is the name of the type
+        and `args` is a sequence of parameters of the type.
+
+        Subclass should override to specialize the behavior.
+        By default, this returns `(self.name, ())`.
+        """
+        return self.name, ()
+
     def __repr__(self):
         return self.name
 
@@ -248,6 +259,12 @@ class Callable(Type):
         Returns a tuple of (list of signatures, parameterized)
         """
 
+    def get_call_type_with_literals(self, context, args, kws, literals):
+        """Simliar to .get_call_type() but with extra argument for literals.
+        Default implementation ignores literals and forwards to .get_call_type().
+        """
+        return self.get_call_type(context, args, kws)
+
 
 class DTypeSpec(Type):
     """
