@@ -1,8 +1,10 @@
 from __future__ import print_function, division, absolute_import
 
+import numpy as np
+
 from collections import namedtuple
 
-from numba import types
+from numba import types, utils
 from numba.typing.templates import (AttributeTemplate, AbstractTemplate,
                                     infer, infer_getattr, signature,
                                     bound_function)
@@ -206,6 +208,14 @@ class SetItemBuffer(AbstractTemplate):
                     return
         elif not isinstance(val, types.Array):
             # Single item assignment
+            if not self.context.can_convert(val, res):
+                # if the array dtype is not yet defined
+                if not res.is_precise():
+                    # set the array type to use the dtype of value (RHS)
+                    newary = ary.copy(dtype=val)
+                    return signature(types.none, newary, idx, res)
+                else:
+                    return
             res = val
         else:
             return
@@ -418,6 +428,22 @@ class ArrayAttribute(AttributeTemplate):
         assert not args
         return signature(ary.copy(ndim=1, layout='C'))
 
+    @bound_function("array.take")
+    def resolve_take(self, ary, args, kws):
+        assert not kws
+        argty, = args
+        if isinstance(argty, types.Integer):
+            sig = signature(ary.dtype, *args)
+        elif isinstance(argty, types.Array):
+            sig = signature(argty.copy(layout='C', dtype=ary.dtype), *args)
+        elif isinstance(argty, types.List): # 1d lists only
+            sig = signature(types.Array(ary.dtype, 1, 'C'), *args)
+        elif isinstance(argty, types.BaseTuple):
+            sig = signature(types.Array(ary.dtype, np.ndim(argty), 'C'), *args)
+        else:
+            raise TypeError("take(%s) not supported for %s" % argty)
+        return sig
+
     def generic_resolve(self, ary, attr):
         # Resolution of other attributes, for record arrays
         if isinstance(ary.dtype, types.Record):
@@ -425,6 +451,23 @@ class ArrayAttribute(AttributeTemplate):
                 return ary.copy(dtype=ary.dtype.typeof(attr), layout='A')
 
 
+@infer_getattr
+class DTypeAttr(AttributeTemplate):
+    key = types.DType
+
+    def resolve_type(self, ary):
+        # Wrap the numeric type in NumberClass
+        return types.NumberClass(ary.dtype)
+
+    def resolve_kind(self, ary):
+        if isinstance(ary.key, types.scalars.Float):
+            val = 'f'
+        elif isinstance(ary.key, types.scalars.Integer):
+            val = 'i'
+        else:
+            return None  # other types not supported yet
+        return types.Const(val)
+
 @infer
 class StaticGetItemArray(AbstractTemplate):
     key = "static_getitem"
@@ -519,9 +562,40 @@ def generic_homog(self, args, kws):
     return signature(self.this.dtype, recvr=self.this)
 
 def generic_expand(self, args, kws):
+    assert not args
+    assert not kws
     return signature(_expand_integer(self.this.dtype), recvr=self.this)
 
+def sum_expand(self, args, kws):
+    """
+    sum can be called with or without an axis parameter.
+    """
+    pysig = None
+    if kws:
+        def sum_stub(axis):
+            pass
+        pysig = utils.pysignature(sum_stub)
+        # rewrite args
+        args = list(args) + [kws['axis']]
+        kws = None
+    args_len = len(args)
+    assert args_len <= 1
+    if args_len == 0:
+        # No axis parameter so the return type of the summation is a scalar
+        # of the type of the array.
+        out = signature(_expand_integer(self.this.dtype), *args,
+                        recvr=self.this)
+    else:
+        # There is an axis paramter so the return type of this summation is
+        # an array of dimension one less than the input array.
+        return_type = types.Array(dtype=_expand_integer(self.this.dtype),
+                                  ndim=self.this.ndim-1, layout='C')
+        out = signature(return_type, *args, recvr=self.this)
+    return out.replace(pysig=pysig)
+
 def generic_expand_cumulative(self, args, kws):
+    assert not args
+    assert not kws
     assert isinstance(self.this, types.Array)
     return_type = types.Array(dtype=_expand_integer(self.this.dtype),
                               ndim=1, layout='C')
@@ -539,10 +613,11 @@ def generic_index(self, args, kws):
     assert not kws
     return signature(types.intp, recvr=self.this)
 
-def install_array_method(name, generic):
+def install_array_method(name, generic, support_literals=False):
     my_attr = {"key": "array." + name, "generic": generic}
     temp_class = type("Array_" + name, (AbstractTemplate,), my_attr)
-
+    if support_literals:
+        temp_class.support_literals = support_literals
     def array_attribute_attachment(self, ary):
         return types.BoundFunction(temp_class, ary)
 
@@ -553,8 +628,8 @@ for fname in ["min", "max"]:
     install_array_method(fname, generic_homog)
 
 # Functions that return a machine-width type, to avoid overflows
-for fname in ["sum", "prod"]:
-    install_array_method(fname, generic_expand)
+install_array_method("prod", generic_expand)
+install_array_method("sum", sum_expand, support_literals=True)
 
 # Functions that return a machine-width type, to avoid overflows
 for fname in ["cumsum", "cumprod"]:
