@@ -173,7 +173,7 @@ class BaseContext(object):
 
         return '\n'.join(desc)
 
-    def resolve_function_type(self, func, args, kws):
+    def resolve_function_type(self, func, args, kws, literals=None):
         """
         Resolve function type *func* for argument types *args* and *kws*.
         A signature is returned.
@@ -184,6 +184,8 @@ class BaseContext(object):
             if functy is not None:
                 func = functy
         if func in self._functions:
+            # Note: Duplicating code with types.Function.get_call_type().
+            #       *defns* are CallTemplates.
             defns = self._functions[func]
             for defn in defns:
                 res = defn.apply(args, kws)
@@ -199,7 +201,7 @@ class BaseContext(object):
 
         if isinstance(func, types.Callable):
             # XXX fold this into the __call__ attribute logic?
-            return func.get_call_type(self, args, kws)
+            return func.get_call_type_with_literals(self, args, kws, literals)
 
     def _get_attribute_templates(self, typ):
         """
@@ -444,7 +446,7 @@ class BaseContext(object):
             else:
                 return min(forward, backward)
 
-    def _rate_arguments(self, actualargs, formalargs):
+    def _rate_arguments(self, actualargs, formalargs, unsafe_casting=True):
         """
         Rate the actual arguments for compatibility against the formal
         arguments.  A Rating instance is returned, or None if incompatible.
@@ -456,6 +458,8 @@ class BaseContext(object):
             conv = self.can_convert(actual, formal)
             if conv is None:
                 return None
+            elif not unsafe_casting and conv >= Conversion.unsafe:
+                return None
 
             if conv == Conversion.promote:
                 rate.promote += 1
@@ -490,20 +494,24 @@ class BaseContext(object):
         return True
 
     def resolve_overload(self, key, cases, args, kws,
-                         allow_ambiguous=True):
+                         allow_ambiguous=True, unsafe_casting=True):
         """
         Given actual *args* and *kws*, find the best matching
         signature in *cases*, or None if none matches.
         *key* is used for error reporting purposes.
         If *allow_ambiguous* is False, a tie in the best matches
         will raise an error.
+        If *unsafe_casting* is False, unsafe casting is forbidden.
         """
         assert not kws, "Keyword arguments are not supported, yet"
+        options = {
+            'unsafe_casting': unsafe_casting,
+        }
         # Rate each case
         candidates = []
         for case in cases:
             if len(args) == len(case.args):
-                rating = self._rate_arguments(args, case.args)
+                rating = self._rate_arguments(args, case.args, **options)
                 if rating is not None:
                     candidates.append((rating.astuple(), case))
 
