@@ -31,6 +31,17 @@ class Signature(object):
         self.recvr = recvr
         self.pysig = pysig
 
+    def replace(self, **kwargs):
+        """Copy and replace the given attributes provided as keyword arguments.
+        Returns an updated copy.
+        """
+        curstate = dict(return_type=self.return_type,
+                        args=self.args,
+                        recvr=self.recvr,
+                        pysig=self.pysig)
+        curstate.update(kwargs)
+        return Signature(**curstate)
+
     def __getstate__(self):
         """
         Needed because of __slots__.
@@ -152,11 +163,22 @@ def fold_arguments(pysig, args, kws, normal_handler, default_handler,
 
 
 class FunctionTemplate(object):
+    # Set to true to disable unsafe cast.
+    # subclass overide-able
+    unsafe_casting = True
+
+    # Whether the typing support literals
+    support_literals = False
+
     def __init__(self, context):
         self.context = context
 
     def _select(self, cases, args, kws):
-        selected = self.context.resolve_overload(self.key, cases, args, kws)
+        options = {
+            'unsafe_casting': self.unsafe_casting,
+        }
+        selected = self.context.resolve_overload(self.key, cases, args, kws,
+                                                 **options)
         return selected
 
     def get_impl_key(self, sig):
@@ -339,7 +361,7 @@ class _IntrinsicTemplate(AbstractTemplate):
         try:
             return self._impl_cache[cache_key]
         except KeyError:
-            result = self._definition_func(self, *args, **kws)
+            result = self._definition_func(self.context, *args, **kws)
             if result is None:
                 return
             [sig, imp] = result
@@ -504,6 +526,10 @@ class _OverloadMethodTemplate(_OverloadAttributeTemplate):
             disp_type = types.Dispatcher(disp)
             sig = disp_type.get_call_type(typing_context, sig.args, {})
             call = context.get_function(disp_type, sig)
+            # Link dependent library
+            cg = context.codegen()
+            for lib in getattr(call, 'libs', ()):
+                cg.add_linking_library(lib)
             return call(builder, args)
 
     def _resolve(self, typ, attr):
